@@ -115,7 +115,7 @@ def lossy_unicode(s):
     except UnicodeEncodeError:
         pass
     return s.decode('latin-1', 'ignore')
-    
+
 
 def output_complete_command(cmdname, args, description, output_list):
     comps = ['complete -c', cmdname]
@@ -129,11 +129,14 @@ def built_command(options, description):
 #    print "Options are: ", options
     man_optionlist = re.split(" |,|\"|=|[|]", options)
     fish_options = []
-    for option in man_optionlist:
-        option = option.strip()
+    for optionstr in man_optionlist:
+        option = re.sub(r"(\[.*\])", "", optionstr)
+        option = option.strip(" \t\r\n[](){}.,:!")
+
 
         # Skip some problematic cases
         if option in ['-', '--']: continue
+        if any(c in "{}()" for c in option): continue
 
         if option.startswith('--'):
             # New style long option (--recursive)
@@ -155,37 +158,37 @@ def built_command(options, description):
     if not fish_options: return
 
     # Here's what we'll use to truncate if necessary
-    max_description_width = 63
+    max_description_width = 78
     if IS_PY3:
-        truncation_suffix = '… [See Man Page]'
+        truncation_suffix = '…'
     else:
         ELLIPSIS_CODE_POINT = 0x2026
-        truncation_suffix = unichr(ELLIPSIS_CODE_POINT) + unicode(' [See Man Page]')
-    
+        truncation_suffix = unichr(ELLIPSIS_CODE_POINT)
+
     # Try to include as many whole sentences as will fit
     # Clean up some probably bogus escapes in the process
     clean_desc = description.replace("\\'", "'").replace("\\.", ".")
     sentences = clean_desc.split('.')
-    
+
     # Clean up "sentences" that are just whitespace
     # But don't let it be empty
     sentences = [x for x in sentences if x.strip()]
     if not sentences: sentences = ['']
 
     udot = lossy_unicode('.')
-    uspace = lossy_unicode(' ') 
-    
-    truncated_description = lossy_unicode(sentences[0]) + udot 
+    uspace = lossy_unicode(' ')
+
+    truncated_description = lossy_unicode(sentences[0]) + udot
     for line in sentences[1:]:
         if not line: continue
-        proposed_description = lossy_unicode(truncated_description) + uspace + lossy_unicode(line) + udot 
+        proposed_description = lossy_unicode(truncated_description) + uspace + lossy_unicode(line) + udot
         if len(proposed_description) <= max_description_width:
             # It fits
             truncated_description = proposed_description
         else:
             # No fit
             break
-    
+
     # If the first sentence does not fit, truncate if necessary
     if len(truncated_description) > max_description_width:
         prefix_len = max_description_width - len(truncation_suffix)
@@ -221,16 +224,13 @@ def remove_groff_formatting(data):
     data = data.replace("\f","")
     return data
 
-class ManParser:
+class ManParser(object):
     def is_my_type(self, manpage):
         return False
 
     def parse_man_page(self, manpage):
         return False
 
-    def name(self):
-        return "no-name"
-
 class Type1ManParser(ManParser):
     def is_my_type(self, manpage):
         #   print manpage
@@ -250,7 +250,7 @@ class Type1ManParser(ManParser):
         options_parts_regex = re.compile("\.PP(.*?)\.RE", re.DOTALL)
         options_matched = re.search(options_parts_regex, options_section)
         #   print options_matched
-        add_diagnostic('Command is ' + CMDNAME)
+        add_diagnostic("Command is %r" % CMDNAME)
 
         if options_matched == None:
             add_diagnostic('Unable to find options')
@@ -272,7 +272,7 @@ class Type1ManParser(ManParser):
                 optionName = data[0].strip()
 
                 if ( optionName.find("-") == -1):
-                    add_diagnostic(optionName + " doesn't contain - ")
+                    add_diagnostic("%r doesn't contain '-' " % optionName)
                 else:
                     optionName = unquote_double_quotes(optionName)
                     optionName = unquote_single_quotes(optionName)
@@ -287,7 +287,7 @@ class Type1ManParser(ManParser):
             options_matched = re.search(options_parts_regex, options_section)
 
     def fallback(self, options_section):
-        add_diagnostic('Falling Back')
+        add_diagnostic('Trying fallback')
         options_parts_regex = re.compile("\.TP( \d+)?(.*?)\.TP", re.DOTALL)
         options_matched = re.search(options_parts_regex, options_section)
         if options_matched == None:
@@ -301,7 +301,7 @@ class Type1ManParser(ManParser):
             if (len(data)>1 and len(data[1].strip())>0): # and len(data[1])<400):
                 optionName = data[0].strip()
                 if ( optionName.find("-") == -1):
-                    add_diagnostic(optionName + "doesn't contains -")
+                    add_diagnostic("%r doesn't contain '-'" % optionName)
                 else:
                     optionName = unquote_double_quotes(optionName)
                     optionName = unquote_single_quotes(optionName)
@@ -316,7 +316,7 @@ class Type1ManParser(ManParser):
         return True
 
     def fallback2(self, options_section):
-        add_diagnostic('Falling Back2')
+        add_diagnostic('Trying last chance fallback')
         ix_remover_regex = re.compile("\.IX.*")
         trailing_num_regex = re.compile('\\d+$')
         options_parts_regex = re.compile("\.IP (.*?)\.IP", re.DOTALL)
@@ -324,7 +324,7 @@ class Type1ManParser(ManParser):
         options_section = re.sub(ix_remover_regex, "", options_section)
         options_matched = re.search(options_parts_regex, options_section)
         if options_matched == None:
-            add_diagnostic('Still not found2')
+            add_diagnostic('Still (still!) not found')
             return False
         while options_matched != None:
             data = options_matched.group(1)
@@ -336,7 +336,7 @@ class Type1ManParser(ManParser):
                 optionName = re.sub(trailing_num_regex, "", data[0].strip())
 
                 if ('-' not in optionName):
-                    add_diagnostic(optionName + " doesn't contain -")
+                    add_diagnostic("%r doesn't contain '-'" % optionName)
                 else:
                     optionName = optionName.strip()
                     optionName = unquote_double_quotes(optionName)
@@ -351,10 +351,6 @@ class Type1ManParser(ManParser):
             options_matched = re.search(options_parts_regex, options_section)
         return True
 
-    def name(self):
-        return "Type1"
-
-
 class Type2ManParser(ManParser):
     def is_my_type(self, manpage):
         options_section_matched = compile_and_search("\.SH OPTIONS(.*?)", manpage)
@@ -370,12 +366,12 @@ class Type2ManParser(ManParser):
 
         options_section = options_section_matched.group(1)
 
-        options_parts_regex = re.compile("\.[I|T]P( \d+(\.\d)?i?)?(.*?)\.[I|T]P", re.DOTALL)
+        options_parts_regex = re.compile("\.[I|T]P( \d+(\.\d)?i?)?(.*?)\.([I|T]P|UNINDENT)", re.DOTALL)
         options_matched = re.search(options_parts_regex, options_section)
-        add_diagnostic('Command is ' + CMDNAME)
+        add_diagnostic('Command is %r' % CMDNAME)
 
         if options_matched == None:
-            add_diagnostic(self.name() + ': Unable to find options')
+            add_diagnostic("%r: Unable to find options" % self)
             return False
 
         while (options_matched != None):
@@ -389,7 +385,7 @@ class Type2ManParser(ManParser):
             if (len(data)>1 and len(data[1].strip())>0): # and len(data[1])<400):
                 optionName = data[0].strip()
                 if '-' not in optionName:
-                    add_diagnostic(optionName + " doesn't contain -")
+                    add_diagnostic("%r doesn't contain '-'" % optionName)
                 else:
                     optionName = unquote_double_quotes(optionName)
                     optionName = unquote_single_quotes(optionName)
@@ -401,9 +397,6 @@ class Type2ManParser(ManParser):
             options_section = options_section[options_matched.end()-3:]
             options_matched = re.search(options_parts_regex, options_section)
 
-    def name(self):
-        return "Type2"
-
 
 class Type3ManParser(ManParser):
     def is_my_type(self, manpage):
@@ -421,7 +414,7 @@ class Type3ManParser(ManParser):
         options_section = options_section_matched.group(1)
         options_parts_regex = re.compile("\.TP(.*?)\.TP", re.DOTALL)
         options_matched = re.search(options_parts_regex, options_section)
-        add_diagnostic('Command is ' + CMDNAME)
+        add_diagnostic("Command is %r" % CMDNAME)
 
         if options_matched == None:
             add_diagnostic('Unable to find options section')
@@ -437,7 +430,7 @@ class Type3ManParser(ManParser):
             if (len(data)>1): # and len(data[1])<400):
                 optionName = data[0].strip()
                 if ( optionName.find("-") == -1):
-                    add_diagnostic(optionName + "doesn't contain -")
+                    add_diagnostic("%r doesn't contain '-'" % optionName)
                 else:
                     optionName = unquote_double_quotes(optionName)
                     optionName = unquote_single_quotes(optionName)
@@ -452,10 +445,6 @@ class Type3ManParser(ManParser):
             options_matched = re.search(options_parts_regex, options_section)
 
 
-    def name(self):
-        return "Type3"
-
-
 class Type4ManParser(ManParser):
     def is_my_type(self, manpage):
         options_section_matched = compile_and_search("\.SH FUNCTION LETTERS(.*?)", manpage)
@@ -472,7 +461,7 @@ class Type4ManParser(ManParser):
         options_section = options_section_matched.group(1)
         options_parts_regex = re.compile("\.TP(.*?)\.TP", re.DOTALL)
         options_matched = re.search(options_parts_regex, options_section)
-        add_diagnostic('Command is ' + CMDNAME)
+        add_diagnostic("Command is %r" % CMDNAME)
 
         if options_matched == None:
             print >> sys.stderr, "Unable to find options section"
@@ -488,7 +477,7 @@ class Type4ManParser(ManParser):
             if (len(data)>1): # and len(data[1])<400):
                 optionName = data[0].strip()
                 if ( optionName.find("-") == -1):
-                    add_diagnostic(optionName + " doesn't contain - ")
+                    add_diagnostic("%r doesn't contain '-' " % optionName)
                 else:
                     optionName = unquote_double_quotes(optionName)
                     optionName = unquote_single_quotes(optionName)
@@ -504,9 +493,6 @@ class Type4ManParser(ManParser):
 
         return True
 
-    def name(self):
-        return "Type4"
-
 class TypeDarwinManParser(ManParser):
     def is_my_type(self, manpage):
         options_section_matched = compile_and_search("\.S[hH] DESCRIPTION", manpage)
@@ -519,7 +505,7 @@ class TypeDarwinManParser(ManParser):
         # Skip leading groff crud
         while re.match('[A-Z][a-z]\s', line):
             line = line[3:]
-            
+
         # If the line ends with a space and then a period or comma, then erase the space
         # This hack handles lines of the form '.Ar projectname .'
         if line.endswith(' ,') or line.endswith(' .'):
@@ -543,7 +529,6 @@ class TypeDarwinManParser(ManParser):
         line = line.replace('.Nm', CMDNAME)
         line = line.replace('\\ ', ' ')
         line = line.replace('\& ', '')
-        line = line.replace(r'.\"', '')
         return line
 
     def is_option(self, line):
@@ -566,10 +551,10 @@ class TypeDarwinManParser(ManParser):
 
             # Get the line and clean it up
             line = lines.pop(0)
-            
+
             # Try to guess how many dashes this argument has
             dash_count = self.count_argument_dashes(line)
-                        
+
             line = self.groff_replace_escapes(line)
             line = self.trim_groff(line)
             line = line.strip()
@@ -582,6 +567,9 @@ class TypeDarwinManParser(ManParser):
             desc_lines = []
             while lines and not self.is_option(lines[0]):
                 line = lossy_unicode(lines.pop(0).strip())
+                # Ignore comments
+                if line.startswith(r'.\"'):
+                    continue
                 if line.startswith('.'):
                     line = self.groff_replace_escapes(line)
                     line = self.trim_groff(line).strip()
@@ -602,9 +590,6 @@ class TypeDarwinManParser(ManParser):
 
         return got_something
 
-    def name(self):
-        return "Darwin man parser"
-
 
 class TypeDeroffManParser(ManParser):
     def is_my_type(self, manpage):
@@ -657,14 +642,11 @@ class TypeDeroffManParser(ManParser):
 
         return got_something
 
-    def name(self):
-        return "Deroffing man parser"
-
 # Return whether the file at the given path is overwritable
 # Raises IOError if it cannot be opened
 def file_is_overwritable(path):
     result = False
-    file = open(path, 'r')
+    file = codecs.open(path, "r", encoding="utf-8")
     for line in file:
         # Skip leading empty lines
         line = line.strip()
@@ -682,7 +664,7 @@ def file_is_overwritable(path):
 
     file.close()
     return result
-    
+
 # Remove any and all autogenerated completions in the given directory
 def cleanup_autogenerated_completions_in_directory(dir):
     try:
@@ -690,14 +672,7 @@ def cleanup_autogenerated_completions_in_directory(dir):
             # Skip non .fish files
             if not filename.endswith('.fish'): continue
             path = os.path.join(dir, filename)
-            try:
-                if file_is_overwritable(path):
-                    os.unlink(path)
-            except IOError:
-                pass
-            except OSError:
-                pass
-                
+            cleanup_autogenerated_file(path)
     except OSError as err:
         return False
 
@@ -736,12 +711,14 @@ def parse_manpage_at_path(manpage_path, output_directory):
         fd = lzma.LZMAFile(str(manpage_path), 'r')
         manpage = fd.read()
         if IS_PY3: manpage = manpage.decode('latin-1')
-    else:
+    elif manpage_path.endswith((".1", ".2", ".3", ".4", ".5", ".6", ".7", ".8", ".9")):
         if IS_PY3:
             fd = open(manpage_path, 'r', encoding='latin-1')
         else:
             fd = open(manpage_path, 'r')
         manpage = fd.read()
+    else:
+        return
     fd.close()
 
     manpage = str(manpage)
@@ -759,7 +736,7 @@ def parse_manpage_at_path(manpage_path, output_directory):
             return
 
     # Ignore the millions of links to BUILTIN(1)
-    if manpage.find('BUILTIN 1') != -1:
+    if 'BUILTIN 1' in manpage or 'builtin.1' in manpage:
         return
 
     # Clear the output list
@@ -776,8 +753,7 @@ def parse_manpage_at_path(manpage_path, output_directory):
         add_diagnostic(manpage_path + ": Not supported")
     else:
         for parser in parsersToTry:
-            parser_name = parser.name()
-            add_diagnostic('Trying parser ' + parser_name)
+            add_diagnostic('Trying %s' % parser.__class__.__name__)
             diagnostic_indent += 1
             success = parser.parse_man_page(manpage)
             diagnostic_indent -= 1
@@ -785,7 +761,7 @@ def parse_manpage_at_path(manpage_path, output_directory):
             if not built_command_output:
                 success = False
             if success:
-                PARSER_INFO.setdefault(parser_name, []).append(CMDNAME)
+                PARSER_INFO.setdefault(parser.__class__.__name__, []).append(CMDNAME)
                 break
 
         if success:
@@ -803,7 +779,8 @@ def parse_manpage_at_path(manpage_path, output_directory):
 
             # Output the magic word Autogenerated so we can tell if we can overwrite this
             built_command_output.insert(1, "# Autogenerated from man page " + manpage_path)
-            built_command_output.insert(2, "# using " + parser_name)
+            # built_command_output.insert(2, "# using " + parser.__class__.__name__) # XXX MISATTRIBUTES THE CULPABILE PARSER! Was really using Type2 but reporting TypeDeroffManParser
+
             for line in built_command_output:
                 output_file.write(line)
                 output_file.write('\n')
@@ -812,18 +789,10 @@ def parse_manpage_at_path(manpage_path, output_directory):
             if output_file != sys.stdout:
                 output_file.close()
         else:
-            parser_names =  ', '.join(p.name() for p in parsersToTry)
+            parser_names =  ', '.join(p.__class__.__name__ for p in parsersToTry)
             #add_diagnostic('%s contains no options or is unparsable' % manpage_path, BRIEF_VERBOSE)
             add_diagnostic('%s contains no options or is unparsable (tried parser %s)' % (manpage_path, parser_names), BRIEF_VERBOSE)
-            # Make sure we delete any old completion
-            if not WRITE_TO_STDOUT:
-                fullpath = os.path.join(output_directory, CMDNAME + '.fish')
-                try:
-                    os.remove(fullpath)
-                except (OSError, IOError):
-                    # Ignore failure
-                    pass
-                
+
     return success
 
 def parse_and_output_man_pages(paths, output_directory, show_progress):
@@ -891,12 +860,27 @@ def parse_and_output_man_pages(paths, output_directory, show_progress):
 def get_paths_from_manpath():
     # Return all the paths to man(1) and man(8) files in the manpath
     import subprocess, os
-    proc = subprocess.Popen(['manpath'], stdout=subprocess.PIPE)
-    manpath, err_data = proc.communicate()
-    parent_paths = manpath.decode().strip().split(':')
-    if not parent_paths:
-        sys.stderr.write("Unable to get the manpath (tried manpath)\n")
-        sys.exit(-1)
+    proc = None
+    # $MANPATH takes precedence, just like with `man` on the CLI.
+    if os.getenv("MANPATH"):
+        parent_paths = os.getenv("MANPATH").strip().split(':')
+    else:
+        # Some systems have manpath, others have `man --path` (like Haiku).
+        # TODO: Deal with systems that have neither (OpenBSD)
+        for prog in [['manpath'], ['man', '--path']]:
+            try:
+                proc = subprocess.Popen(prog, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            except OSError: # Command does not exist, keep trying
+                continue
+            break # Command exists, use it.
+        manpath, err_data = proc.communicate()
+        parent_paths = manpath.decode().strip().split(':')
+    if (not parent_paths) or (proc and proc.returncode > 0):
+        # HACK: Use some fallback in case we can't get anything else.
+        # `mandoc` does not provide `manpath` or `man --path` and $MANPATH might not be set, so just use the default for mandoc (minus /usr/X11R6/man, because that's not relevant).
+        # The alternative is reading its config file (/etc/man.conf)
+        sys.stderr.write("Unable to get the manpath, falling back to /usr/share/man:/usr/local/share/man. Please set $MANPATH if that is not correct.\n")
+        parent_paths = ["/usr/share/man", "/usr/local/share/man"]
     result = []
     for parent_path in parent_paths:
         for section in ['man1', 'man6', 'man8']:
@@ -916,7 +900,7 @@ def usage(script_name):
      -h, --help\t\tShow this help message
      -v, --verbose [0, 1, 2]\tShow debugging output to stderr. Larger is more verbose.
      -s, --stdout\tWrite all completions to stdout (trumps the --directory option)
-     -d, --directory [dir]\tWrite all completions to the given directory, instead of to ~/.config/fish/generated_completions
+     -d, --directory [dir]\tWrite all completions to the given directory, instead of to ~/.local/share/fish/generated_completions
      -m, --manpath\tProcess all man1 and man8 files available in the manpath (as determined by manpath)
      -p, --progress\tShow progress
     """)
@@ -929,7 +913,7 @@ if __name__ == "__main__":
         print(err.msg) # will print something like "option -a not recognized"
         usage(script_name)
         sys.exit(2)
-        
+
     # Directories within which we will clean up autogenerated completions
     # This script originally wrote completions into ~/.config/fish/completions
     # Now it writes them into a separate directory
@@ -969,9 +953,9 @@ if __name__ == "__main__":
     if not file_paths:
         print("No paths specified")
         sys.exit(0)
-        
+
     if not WRITE_TO_STDOUT and not output_directory:
-        # Default to ~/.config/fish/generated_completions/
+        # Default to ~/.local/share/fish/generated_completions/
         # Create it if it doesn't exist
         xdg_data_home = os.getenv('XDG_DATA_HOME', '~/.local/share')
         output_directory = os.path.expanduser(xdg_data_home + '/fish/generated_completions/')
@@ -981,4 +965,8 @@ if __name__ == "__main__":
             if e.errno != errno.EEXIST:
                 raise
 
+    if not WRITE_TO_STDOUT:
+        # Remove old generated files
+        cleanup_autogenerated_completions_in_directory(output_directory)
+
     parse_and_output_man_pages(file_paths, output_directory, show_progress)
