@@ -74,7 +74,7 @@ function adjust_lightness(color_str, func) {
     if (color_str[0] == '#') {
         color_str = color_str.substring(1);
     }
-	
+
     /* Hack to handle for example F00 */
     if (color_str.length == 3) {
         color_str = color_str[0] + color_str[0] + color_str[1] + color_str[1] + color_str[2] + color_str[2]
@@ -128,7 +128,7 @@ function text_color_for_color(color_str) {
     return adjust_lightness(color_str, compute_constrast);
 }
 
-function rgb_to_hsl(r, g, b) {	
+function rgb_to_hsl(r, g, b) {
     r /= 255, g /= 255, b /= 255;
     var max = Math.max(r, g, b), min = Math.min(r, g, b);
     var h, s, l = (max + min) / 2;
@@ -243,12 +243,12 @@ var color_scheme_fish_default = {
 var TomorrowTheme = {
     tomorrow_night: {'Background': '1d1f21', 'Current Line': '282a2e', 'Selection': '373b41', 'Foreground': 'c5c8c6', 'Comment': '969896', 'Red': 'cc6666', 'Orange': 'de935f',  'Yellow': 'f0c674', 'Green': 'b5bd68', 'Aqua': '8abeb7', 'Blue': '81a2be', 'Purple': 'b294bb'
     },
-    
+
     tomorrow: {'Background': 'ffffff', 'Current Line': 'efefef', 'Selection': 'd6d6d6', 'Foreground': '4d4d4c', 'Comment': '8e908c', 'Red': 'c82829', 'Orange': 'f5871f', 'Yellow': 'eab700', 'Green': '718c00', 'Aqua': '3e999f', 'Blue': '4271ae', 'Purple': '8959a8'
     },
-    
+
     tomorrow_night_bright: {'Background': '000000', 'Current Line': '2a2a2a', 'Selection': '424242', 'Foreground': 'eaeaea', 'Comment': '969896', 'Red': 'd54e53', 'Orange': 'e78c45', 'Yellow': 'e7c547', 'Green': 'b9ca4a', 'Aqua': '70c0b1', 'Blue': '7aa6da', 'Purple': 'c397d8'},
-    
+
     apply: function(theme, receiver){
         receiver['autosuggestion'] = theme['Comment']
         receiver['command'] = theme['Purple']
@@ -258,7 +258,7 @@ var TomorrowTheme = {
         receiver['param'] = theme['Blue']
         receiver['quote'] = theme['Green']
         receiver['redirection'] = theme['Aqua']
-        
+
         receiver['colors'] = []
         for (var key in theme) receiver['colors'].push(theme[key])
     }
@@ -273,9 +273,9 @@ var solarized = {
 var color_scheme_solarized_light = {
     name: "Solarized Light",
     colors: dict_values(solarized),
-    
+
     preferred_background: '#' + solarized.base3,
-    
+
     autosuggestion: solarized.base1,
     command: solarized.base01,
     comment: solarized.base1,
@@ -284,7 +284,7 @@ var color_scheme_solarized_light = {
     param: solarized.base00,
     quote: solarized.base0,
     redirection: solarized.violet,
-    
+
     url: 'http://ethanschoonover.com/solarized'
 };
 
@@ -292,7 +292,7 @@ var color_scheme_solarized_dark = {
     name: "Solarized Dark",
     colors: dict_values(solarized),
     preferred_background: '#' + solarized.base03,
-    
+
     autosuggestion: solarized.base01,
     command: solarized.base1,
     comment: solarized.base01,
@@ -301,7 +301,7 @@ var color_scheme_solarized_dark = {
     param: solarized.base0,
     quote: solarized.base00,
     redirection: solarized.violet,
-    
+
     url: 'http://ethanschoonover.com/solarized'
 };
 
@@ -332,15 +332,15 @@ function construct_scheme_analogous(label, background, color_list) {
         name: label,
         preferred_background: background,
         colors: color_list,
-        
+
         command: color_list[0],
         quote: color_list[6],
         param: color_list[5],
         autosuggestion: color_list[4],
-        
+
         error: color_list[9],
         comment: color_list[12],
-        
+
         end: color_list[10],
         redirection: color_list[11]
     };
@@ -351,16 +351,16 @@ function construct_scheme_triad(label, background, color_list) {
         name: label,
         preferred_background: background,
         colors: color_list,
-        
+
         command: color_list[0],
         quote: color_list[2],
         param: color_list[1],
         autosuggestion: color_list[3],
         redirection: color_list[4],
-        
+
         error: color_list[8],
         comment: color_list[10],
-        
+
         end: color_list[7],
     };
 }
@@ -370,30 +370,30 @@ function construct_scheme_complementary(label, background, color_list) {
         name: label,
         preferred_background: background,
         colors: color_list,
-        
+
         command: color_list[0],
         quote: color_list[4],
         param: color_list[3],
         autosuggestion: color_list[2],
         redirection: color_list[6],
-        
+
         error: color_list[5],
         comment: color_list[8],
-        
+
         end: color_list[9],
     };
 }
 
 function construct_color_scheme_mono(label, background, from_end) {
     var mono_colors = ['000000', '121212', '1c1c1c', '262626', '303030', '3a3a3a', '444444', '4e4e4e', '585858', '5f5f5f', '626262', '6c6c6c', '767676', '808080', '878787', '8a8a8a', '949494', '9e9e9e', 'a8a8a8', 'afafaf', 'b2b2b2', 'bcbcbc', 'c6c6c6', 'd0d0d0', 'd7d7d7', 'dadada', 'e4e4e4', 'eeeeee', 'ffffff'];
-    
+
     if (from_end) mono_colors.reverse();
-    
+
     return {
         name: label,
         preferred_background: background,
         colors: mono_colors,
-        
+
         autosuggestion: '777777',
         command: mono_colors[0],
         comment: mono_colors[7],
@@ -407,21 +407,21 @@ function construct_color_scheme_mono(label, background, from_end) {
 
 var additional_color_schemes = [
     construct_scheme_analogous('Snow Day', 'white', ['164CC9', '325197', '072D83', '4C7AE4', '7596E4', '4319CC', '4C3499', '260885', '724EE5', '9177E5', '02BDBD', '248E8E', '007B7B', '39DEDE', '65DEDE']),
-    
+
     construct_scheme_analogous('Lava', '#232323', ['FF9400', 'BF8330', 'A66000', 'FFAE40', 'FFC473', 'FFC000', 'BF9C30', 'A67D00', 'FFD040', 'FFDD73', 'FF4C00', 'BF5B30', 'A63100', 'FF7940', 'FF9D73']),
-    
+
     construct_scheme_analogous('Seaweed', '#232323', ['00BF32', '248F40', '007C21', '38DF64', '64DF85', '04819E', '206676', '015367', '38B2CE', '60B9CE', '8EEB00', '7CB02C', '5C9900', 'ACF53D', 'C0F56E']),
-    
+
     construct_scheme_triad('Fairground', '#003', ['0772A1', '225E79', '024A68', '3BA3D0', '63AFD0', 'D9005B', 'A3295C', '8D003B', 'EC3B86', 'EC6AA1', 'FFE100', 'BFAE30', 'A69200', 'FFE840', 'FFEE73']),
-    
+
     construct_scheme_complementary('Bay Cruise', 'black', ['009999', '1D7373', '006363', '33CCCC', '5CCCCC', 'FF7400', 'BF7130', 'A64B00', 'FF9640', 'FFB273']),
 
 {
     'name': 'Old School',
     'preferred_background': 'black',
-    
+
     colors: ['00FF00', '30BE30', '00A400', '44FF44', '7BFF7B', 'FF0000', 'BE3030', 'A40000', 'FF7B7B', '777777', 'CCCCCC'],
-    
+
     autosuggestion: '777777',
     command: '00FF00',
     comment: '30BE30',
@@ -435,9 +435,9 @@ var additional_color_schemes = [
 {
     'name': 'Just a Touch',
     'preferred_background': 'black',
-    
+
     colors: ['B0B0B0', '969696', '789276', 'F4F4F4', 'A0A0F0', '666A80', 'F0F0F0', 'D7D7D7', 'B7B7B7', 'FFA779', 'FAFAFA'],
-    
+
     autosuggestion: '9C9C9C',
     command: 'F4F4F4',
     comment: 'B0B0B0',
@@ -448,6 +448,22 @@ var additional_color_schemes = [
     redirection: 'FAFAFA'
 },
 
+{
+    'name': 'Dracula',
+    'preferred_background': '#282a36',
+
+    colors: ['282A36', '44475A', '44475A', 'F8F8F2', '6272A4', '8BE9FD', '50FA7B', 'FFB86C', 'FF79C6', 'BD93F9', 'FF5555', 'F1FA8C'],
+
+    autosuggestion: 'BD93F9',
+    command: 'F8F8F2',
+    comment: '6272A4',
+    end: '50FA7B',
+    error: 'FFB86C',
+    param: 'FF79C6',
+    quote: 'F1FA8C',
+    redirection: '8BE9FD'
+},
+
 construct_color_scheme_mono('Mono Lace', 'white', false),
 construct_color_scheme_mono('Mono Smoke', 'black', true)
 ];
