@@ -3,23 +3,24 @@
 // Implemented from scratch (yes, really) by way of IEEE 1003.1 as reference.
 #include "config.h"  // IWYU pragma: keep
 
-#include <assert.h>
+#include <errno.h>
 #include <stdarg.h>
-#include <stdio.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <wchar.h>
+
 #include <memory>
 #include <string>
+#include <type_traits>
 
 #include "builtin.h"
 #include "common.h"
 #include "io.h"
-#include "proc.h"
 #include "wutil.h"  // IWYU pragma: keep
 
-enum { BUILTIN_TEST_SUCCESS = STATUS_BUILTIN_OK, BUILTIN_TEST_FAIL = STATUS_BUILTIN_ERROR };
+using std::unique_ptr;
+using std::move;
 
 int builtin_test(parser_t &parser, io_streams_t &streams, wchar_t **argv);
 
@@ -38,6 +39,7 @@ enum token_t {
     test_filetype_G,  // "-G", for check effective group id
     test_filetype_g,  // "-g", for set-group-id
     test_filetype_h,  // "-h", for symbolic links
+    test_filetype_k,  // "-k", for sticky bit
     test_filetype_L,  // "-L", same as -h
     test_filetype_O,  // "-O", for check effective user id
     test_filetype_p,  // "-p", for FIFO
@@ -92,6 +94,7 @@ static const struct token_info_t {
                    {test_filetype_G, L"-G", UNARY_PRIMARY},
                    {test_filetype_g, L"-g", UNARY_PRIMARY},
                    {test_filetype_h, L"-h", UNARY_PRIMARY},
+                   {test_filetype_k, L"-k", UNARY_PRIMARY},
                    {test_filetype_L, L"-L", UNARY_PRIMARY},
                    {test_filetype_O, L"-O", UNARY_PRIMARY},
                    {test_filetype_p, L"-p", UNARY_PRIMARY},
@@ -146,7 +149,7 @@ class test_parser {
     wcstring_list_t strings;
     wcstring_list_t errors;
 
-    expression *error(const wchar_t *fmt, ...);
+    unique_ptr<expression> error(const wchar_t *fmt, ...);
     void add_error(const wchar_t *fmt, ...);
 
     const wcstring &arg(unsigned int idx) { return strings.at(idx); }
@@ -154,19 +157,20 @@ class test_parser {
    public:
     explicit test_parser(const wcstring_list_t &val) : strings(val) {}
 
-    expression *parse_expression(unsigned int start, unsigned int end);
-    expression *parse_3_arg_expression(unsigned int start, unsigned int end);
-    expression *parse_4_arg_expression(unsigned int start, unsigned int end);
-    expression *parse_combining_expression(unsigned int start, unsigned int end);
-    expression *parse_unary_expression(unsigned int start, unsigned int end);
+    unique_ptr<expression> parse_expression(unsigned int start, unsigned int end);
+    unique_ptr<expression> parse_3_arg_expression(unsigned int start, unsigned int end);
+    unique_ptr<expression> parse_4_arg_expression(unsigned int start, unsigned int end);
+    unique_ptr<expression> parse_combining_expression(unsigned int start, unsigned int end);
+    unique_ptr<expression> parse_unary_expression(unsigned int start, unsigned int end);
 
-    expression *parse_primary(unsigned int start, unsigned int end);
-    expression *parse_parenthentical(unsigned int start, unsigned int end);
-    expression *parse_unary_primary(unsigned int start, unsigned int end);
-    expression *parse_binary_primary(unsigned int start, unsigned int end);
-    expression *parse_just_a_string(unsigned int start, unsigned int end);
+    unique_ptr<expression> parse_primary(unsigned int start, unsigned int end);
+    unique_ptr<expression> parse_parenthentical(unsigned int start, unsigned int end);
+    unique_ptr<expression> parse_unary_primary(unsigned int start, unsigned int end);
+    unique_ptr<expression> parse_binary_primary(unsigned int start, unsigned int end);
+    unique_ptr<expression> parse_just_a_string(unsigned int start, unsigned int end);
 
-    static expression *parse_args(const wcstring_list_t &args, wcstring &err);
+    static unique_ptr<expression> parse_args(const wcstring_list_t &args, wcstring &err,
+                                             wchar_t *program_name);
 };
 
 struct range_t {
@@ -187,12 +191,10 @@ class expression {
 
     virtual ~expression() {}
 
-    /// Evaluate returns true if the expression is true (i.e. BUILTIN_TEST_SUCCESS).
+    /// Evaluate returns true if the expression is true (i.e. STATUS_CMD_OK).
     virtual bool evaluate(wcstring_list_t &errors) = 0;
 };
 
-typedef std::auto_ptr<expression> expr_ref_t;
-
 /// Single argument like -n foo or "just a string".
 class unary_primary : public expression {
    public:
@@ -216,9 +218,9 @@ class binary_primary : public expression {
 /// Unary operator like bang.
 class unary_operator : public expression {
    public:
-    expr_ref_t subject;
-    unary_operator(token_t tok, range_t where, expr_ref_t &exp)
-        : expression(tok, where), subject(exp) {}
+    unique_ptr<expression> subject;
+    unary_operator(token_t tok, range_t where, unique_ptr<expression> exp)
+        : expression(tok, where), subject(move(exp)) {}
     bool evaluate(wcstring_list_t &errors);
 };
 
@@ -226,22 +228,17 @@ class unary_operator : public expression {
 /// we don't have to worry about precedence in the parser.
 class combining_expression : public expression {
    public:
-    const std::vector<expression *> subjects;
+    const std::vector<unique_ptr<expression>> subjects;
     const std::vector<token_t> combiners;
 
-    combining_expression(token_t tok, range_t where, const std::vector<expression *> &exprs,
+    combining_expression(token_t tok, range_t where, std::vector<unique_ptr<expression>> exprs,
                          const std::vector<token_t> &combs)
-        : expression(tok, where), subjects(exprs), combiners(combs) {
+        : expression(tok, where), subjects(std::move(exprs)), combiners(std::move(combs)) {
         // We should have one more subject than combiner.
         assert(subjects.size() == combiners.size() + 1);
     }
 
-    // We are responsible for destroying our expressions.
-    virtual ~combining_expression() {
-        for (size_t i = 0; i < subjects.size(); i++) {
-            delete subjects[i];
-        }
-    }
+    virtual ~combining_expression() {}
 
     bool evaluate(wcstring_list_t &errors);
 };
@@ -249,9 +246,9 @@ class combining_expression : public expression {
 /// Parenthetical expression.
 class parenthetical_expression : public expression {
    public:
-    expr_ref_t contents;
-    parenthetical_expression(token_t tok, range_t where, expr_ref_t &expr)
-        : expression(tok, where), contents(expr) {}
+    unique_ptr<expression> contents;
+    parenthetical_expression(token_t tok, range_t where, unique_ptr<expression> expr)
+        : expression(tok, where), contents(move(expr)) {}
 
     virtual bool evaluate(wcstring_list_t &errors);
 };
@@ -264,7 +261,7 @@ void test_parser::add_error(const wchar_t *fmt, ...) {
     va_end(va);
 }
 
-expression *test_parser::error(const wchar_t *fmt, ...) {
+unique_ptr<expression> test_parser::error(const wchar_t *fmt, ...) {
     assert(fmt != NULL);
     va_list va;
     va_start(va, fmt);
@@ -273,15 +270,16 @@ expression *test_parser::error(const wchar_t *fmt, ...) {
     return NULL;
 }
 
-expression *test_parser::parse_unary_expression(unsigned int start, unsigned int end) {
+unique_ptr<expression> test_parser::parse_unary_expression(unsigned int start, unsigned int end) {
     if (start >= end) {
         return error(L"Missing argument at index %u", start);
     }
     token_t tok = token_for_string(arg(start))->tok;
     if (tok == test_bang) {
-        expr_ref_t subject(parse_unary_expression(start + 1, end));
+        unique_ptr<expression> subject(parse_unary_expression(start + 1, end));
         if (subject.get()) {
-            return new unary_operator(tok, range_t(start, subject->range.end), subject);
+            return make_unique<unary_operator>(tok, range_t(start, subject->range.end),
+                                               move(subject));
         }
         return NULL;
     }
@@ -289,10 +287,11 @@ expression *test_parser::parse_unary_expression(unsigned int start, unsigned int
 }
 
 /// Parse a combining expression (AND, OR).
-expression *test_parser::parse_combining_expression(unsigned int start, unsigned int end) {
+unique_ptr<expression> test_parser::parse_combining_expression(unsigned int start,
+                                                               unsigned int end) {
     if (start >= end) return NULL;
 
-    std::vector<expression *> subjects;
+    std::vector<unique_ptr<expression>> subjects;
     std::vector<token_t> combiners;
     unsigned int idx = start;
     bool first = true;
@@ -313,7 +312,7 @@ expression *test_parser::parse_combining_expression(unsigned int start, unsigned
         }
 
         // Parse another expression.
-        expression *expr = parse_unary_expression(idx, end);
+        unique_ptr<expression> expr = parse_unary_expression(idx, end);
         if (!expr) {
             add_error(L"Missing argument at index %u", idx);
             if (!first) {
@@ -325,7 +324,7 @@ expression *test_parser::parse_combining_expression(unsigned int start, unsigned
 
         // Go to the end of this expression.
         idx = expr->range.end;
-        subjects.push_back(expr);
+        subjects.push_back(move(expr));
         first = false;
     }
 
@@ -334,10 +333,11 @@ expression *test_parser::parse_combining_expression(unsigned int start, unsigned
     }
     // Our new expression takes ownership of all expressions we created. The token we pass is
     // irrelevant.
-    return new combining_expression(test_combine_and, range_t(start, idx), subjects, combiners);
+    return make_unique<combining_expression>(test_combine_and, range_t(start, idx), move(subjects),
+                                             move(combiners));
 }
 
-expression *test_parser::parse_unary_primary(unsigned int start, unsigned int end) {
+unique_ptr<expression> test_parser::parse_unary_primary(unsigned int start, unsigned int end) {
     // We need two arguments.
     if (start >= end) {
         return error(L"Missing argument at index %u", start);
@@ -350,10 +350,10 @@ expression *test_parser::parse_unary_primary(unsigned int start, unsigned int en
     const token_info_t *info = token_for_string(arg(start));
     if (!(info->flags & UNARY_PRIMARY)) return NULL;
 
-    return new unary_primary(info->tok, range_t(start, start + 2), arg(start + 1));
+    return make_unique<unary_primary>(info->tok, range_t(start, start + 2), arg(start + 1));
 }
 
-expression *test_parser::parse_just_a_string(unsigned int start, unsigned int end) {
+unique_ptr<expression> test_parser::parse_just_a_string(unsigned int start, unsigned int end) {
     // Handle a string as a unary primary that is not a token of any other type. e.g. 'test foo -a
     // bar' should evaluate to true We handle this with a unary primary of test_string_n.
 
@@ -369,48 +369,10 @@ expression *test_parser::parse_just_a_string(unsigned int start, unsigned int en
 
     // This is hackish; a nicer way to implement this would be with a "just a string" expression
     // type.
-    return new unary_primary(test_string_n, range_t(start, start + 1), arg(start));
+    return make_unique<unary_primary>(test_string_n, range_t(start, start + 1), arg(start));
 }
 
-#if 0
-expression *test_parser::parse_unary_primary(unsigned int start, unsigned int end)
-{
-    // We need either one or two arguments.
-    if (start >= end) {
-        return error(L"Missing argument at index %u", start);
-    }
-
-    // The index of the argument to the unary primary.
-    unsigned int arg_idx;
-
-    // All our unary primaries are prefix, so any operator is at start. But it also may just be a
-    // string, with no operator.
-    const token_info_t *info = token_for_string(arg(start));
-    if (info->flags & UNARY_PRIMARY) {
-        /* We have an operator. Skip the operator argument */
-        arg_idx = start + 1;
-
-        // We have some freedom here...do we allow other tokens for the argument to operate on? For
-        // example, should 'test -n =' work? I say yes. So no typechecking on the next token.
-    } else if (info->tok == test_unknown) {
-        // "Just a string.
-        arg_idx = start;
-    } else {
-        // Here we don't allow arbitrary tokens as "just a string." I.e. 'test = -a =' should have a
-        // parse error. We could relax this at some point.
-        return error(L"Parse error at argument index %u", start);
-    }
-
-    // Verify we have the argument we want, i.e. test -n should fail to parse.
-    if (arg_idx >= end) {
-        return error(L"Missing argument at index %u", arg_idx);
-    }
-
-    return new unary_primary(info->tok, range_t(start, arg_idx + 1), arg(arg_idx));
-}
-#endif
-
-expression *test_parser::parse_binary_primary(unsigned int start, unsigned int end) {
+unique_ptr<expression> test_parser::parse_binary_primary(unsigned int start, unsigned int end) {
     // We need three arguments.
     for (unsigned int idx = start; idx < start + 3; idx++) {
         if (idx >= end) {
@@ -422,10 +384,11 @@ expression *test_parser::parse_binary_primary(unsigned int start, unsigned int e
     const token_info_t *info = token_for_string(arg(start + 1));
     if (!(info->flags & BINARY_PRIMARY)) return NULL;
 
-    return new binary_primary(info->tok, range_t(start, start + 3), arg(start), arg(start + 2));
+    return make_unique<binary_primary>(info->tok, range_t(start, start + 3), arg(start),
+                                       arg(start + 2));
 }
 
-expression *test_parser::parse_parenthentical(unsigned int start, unsigned int end) {
+unique_ptr<expression> test_parser::parse_parenthentical(unsigned int start, unsigned int end) {
     // We need at least three arguments: open paren, argument, close paren.
     if (start + 3 >= end) return NULL;
 
@@ -434,9 +397,8 @@ expression *test_parser::parse_parenthentical(unsigned int start, unsigned int e
     if (open_paren->tok != test_paren_open) return NULL;
 
     // Parse a subexpression.
-    expression *subexr_ptr = parse_expression(start + 1, end);
-    if (!subexr_ptr) return NULL;
-    expr_ref_t subexpr(subexr_ptr);
+    unique_ptr<expression> subexpr = parse_expression(start + 1, end);
+    if (!subexpr) return NULL;
 
     // Parse a close paren.
     unsigned close_index = subexpr->range.end;
@@ -450,15 +412,16 @@ expression *test_parser::parse_parenthentical(unsigned int start, unsigned int e
     }
 
     // Success.
-    return new parenthetical_expression(test_paren_open, range_t(start, close_index + 1), subexpr);
+    return make_unique<parenthetical_expression>(test_paren_open, range_t(start, close_index + 1),
+                                                 move(subexpr));
 }
 
-expression *test_parser::parse_primary(unsigned int start, unsigned int end) {
+unique_ptr<expression> test_parser::parse_primary(unsigned int start, unsigned int end) {
     if (start >= end) {
         return error(L"Missing argument at index %u", start);
     }
 
-    expression *expr = NULL;
+    unique_ptr<expression> expr = NULL;
     if (!expr) expr = parse_parenthentical(start, end);
     if (!expr) expr = parse_unary_primary(start, end);
     if (!expr) expr = parse_binary_primary(start, end);
@@ -467,24 +430,24 @@ expression *test_parser::parse_primary(unsigned int start, unsigned int end) {
 }
 
 // See IEEE 1003.1 breakdown of the behavior for different parameter counts.
-expression *test_parser::parse_3_arg_expression(unsigned int start, unsigned int end) {
+unique_ptr<expression> test_parser::parse_3_arg_expression(unsigned int start, unsigned int end) {
     assert(end - start == 3);
-    expression *result = NULL;
+    unique_ptr<expression> result = NULL;
 
     const token_info_t *center_token = token_for_string(arg(start + 1));
     if (center_token->flags & BINARY_PRIMARY) {
         result = parse_binary_primary(start, end);
     } else if (center_token->tok == test_combine_and || center_token->tok == test_combine_or) {
-        expr_ref_t left(parse_unary_expression(start, start + 1));
-        expr_ref_t right(parse_unary_expression(start + 2, start + 3));
+        unique_ptr<expression> left(parse_unary_expression(start, start + 1));
+        unique_ptr<expression> right(parse_unary_expression(start + 2, start + 3));
         if (left.get() && right.get()) {
             // Transfer ownership to the vector of subjects.
-            std::vector<token_t> combiners(1, center_token->tok);
-            std::vector<expression *> subjects;
-            subjects.push_back(left.release());
-            subjects.push_back(right.release());
-            result = new combining_expression(center_token->tok, range_t(start, end), subjects,
-                                              combiners);
+            std::vector<token_t> combiners = {center_token->tok};
+            std::vector<unique_ptr<expression>> subjects;
+            subjects.push_back(move(left));
+            subjects.push_back(move(right));
+            result = make_unique<combining_expression>(center_token->tok, range_t(start, end),
+                                                       move(subjects), move(combiners));
         }
     } else {
         result = parse_unary_expression(start, end);
@@ -492,15 +455,16 @@ expression *test_parser::parse_3_arg_expression(unsigned int start, unsigned int
     return result;
 }
 
-expression *test_parser::parse_4_arg_expression(unsigned int start, unsigned int end) {
+unique_ptr<expression> test_parser::parse_4_arg_expression(unsigned int start, unsigned int end) {
     assert(end - start == 4);
-    expression *result = NULL;
+    unique_ptr<expression> result = NULL;
 
     token_t first_token = token_for_string(arg(start))->tok;
     if (first_token == test_bang) {
-        expr_ref_t subject(parse_3_arg_expression(start + 1, end));
+        unique_ptr<expression> subject(parse_3_arg_expression(start + 1, end));
         if (subject.get()) {
-            result = new unary_operator(first_token, range_t(start, subject->range.end), subject);
+            result = make_unique<unary_operator>(first_token, range_t(start, subject->range.end),
+                                                 move(subject));
         }
     } else if (first_token == test_paren_open) {
         result = parse_parenthentical(start, end);
@@ -510,7 +474,7 @@ expression *test_parser::parse_4_arg_expression(unsigned int start, unsigned int
     return result;
 }
 
-expression *test_parser::parse_expression(unsigned int start, unsigned int end) {
+unique_ptr<expression> test_parser::parse_expression(unsigned int start, unsigned int end) {
     if (start >= end) {
         return error(L"Missing argument at index %u", start);
     }
@@ -518,8 +482,8 @@ expression *test_parser::parse_expression(unsigned int start, unsigned int end)
     unsigned int argc = end - start;
     switch (argc) {
         case 0: {
-            assert(0);  // should have been caught by the above test
-            return NULL;
+            DIE("argc should not be zero");  // should have been caught by the above test
+            break;
         }
         case 1: {
             return error(L"Missing argument at index %u", start + 1);
@@ -537,17 +501,19 @@ expression *test_parser::parse_expression(unsigned int start, unsigned int end)
     }
 }
 
-expression *test_parser::parse_args(const wcstring_list_t &args, wcstring &err) {
+unique_ptr<expression> test_parser::parse_args(const wcstring_list_t &args, wcstring &err,
+                                               wchar_t *program_name) {
     // Empty list and one-arg list should be handled by caller.
     assert(args.size() > 1);
 
     test_parser parser(args);
-    expression *result = parser.parse_expression(0, (unsigned int)args.size());
+    unique_ptr<expression> result = parser.parse_expression(0, (unsigned int)args.size());
 
     // Handle errors.
     // For now we only show the first error.
     if (!parser.errors.empty()) {
-        err.append(L"test: ");
+        err.append(program_name);
+        err.append(L": ");
         err.append(parser.errors.at(0));
         err.push_back(L'\n');
     }
@@ -558,12 +524,10 @@ expression *test_parser::parse_args(const wcstring_list_t &args, wcstring &err)
         assert(result->range.end <= args.size());
         if (result->range.end < args.size()) {
             if (err.empty()) {
-                append_format(err, L"test: unexpected argument at index %lu: '%ls'\n",
+                append_format(err, L"%ls: unexpected argument at index %lu: '%ls'\n", program_name,
                               (unsigned long)result->range.end, args.at(result->range.end).c_str());
             }
-
-            delete result;
-            result = NULL;
+            result.reset(NULL);
         }
     }
 
@@ -579,62 +543,55 @@ bool binary_primary::evaluate(wcstring_list_t &errors) {
 }
 
 bool unary_operator::evaluate(wcstring_list_t &errors) {
-    switch (token) {
-        case test_bang: {
-            assert(subject.get());
-            return !subject->evaluate(errors);
-        }
-        default: {
-            errors.push_back(format_string(L"Unknown token type in %s", __func__));
-            return false;
-        }
+    if (token == test_bang) {
+        assert(subject.get());
+        return !subject->evaluate(errors);
     }
+
+    errors.push_back(format_string(L"Unknown token type in %s", __func__));
+    return false;
 }
 
 bool combining_expression::evaluate(wcstring_list_t &errors) {
-    switch (token) {
-        case test_combine_and:
-        case test_combine_or: {
-            // One-element case.
-            if (subjects.size() == 1) return subjects.at(0)->evaluate(errors);
+    if (token == test_combine_and || token == test_combine_or) {
+        assert(!subjects.empty());  //!OCLINT(multiple unary operator)
+        assert(combiners.size() + 1 == subjects.size());
 
-            // Evaluate our lists, remembering that AND has higher precedence than OR. We can
-            // visualize this as a sequence of OR expressions of AND expressions.
-            assert(combiners.size() + 1 == subjects.size());
-            assert(!subjects.empty());
+        // One-element case.
+        if (subjects.size() == 1) return subjects.at(0)->evaluate(errors);
 
-            size_t idx = 0, max = subjects.size();
-            bool or_result = false;
-            while (idx < max) {
-                if (or_result) {  // short circuit
+        // Evaluate our lists, remembering that AND has higher precedence than OR. We can
+        // visualize this as a sequence of OR expressions of AND expressions.
+        size_t idx = 0, max = subjects.size();
+        bool or_result = false;
+        while (idx < max) {
+            if (or_result) {  // short circuit
+                break;
+            }
+
+            // Evaluate a stream of AND starting at given subject index. It may only have one
+            // element.
+            bool and_result = true;
+            for (; idx < max; idx++) {
+                // Evaluate it, short-circuiting.
+                and_result = and_result && subjects.at(idx)->evaluate(errors);
+
+                // If the combiner at this index (which corresponding to how we combine with the
+                // next subject) is not AND, then exit the loop.
+                if (idx + 1 < max && combiners.at(idx) != test_combine_and) {
+                    idx++;
                     break;
                 }
-
-                // Evaluate a stream of AND starting at given subject index. It may only have one
-                // element.
-                bool and_result = true;
-                for (; idx < max; idx++) {
-                    // Evaluate it, short-circuiting.
-                    and_result = and_result && subjects.at(idx)->evaluate(errors);
-
-                    // If the combiner at this index (which corresponding to how we combine with the
-                    // next subject) is not AND, then exit the loop.
-                    if (idx + 1 < max && combiners.at(idx) != test_combine_and) {
-                        idx++;
-                        break;
-                    }
-                }
-
-                // OR it in.
-                or_result = or_result || and_result;
             }
-            return or_result;
-        }
-        default: {
-            errors.push_back(format_string(L"Unknown token type in %s", __func__));
-            return BUILTIN_TEST_FAIL;
+
+            // OR it in.
+            or_result = or_result || and_result;
         }
+        return or_result;
     }
+
+    errors.push_back(format_string(L"Unknown token type in %s", __func__));
+    return STATUS_INVALID_ARGS;
 }
 
 bool parenthetical_expression::evaluate(wcstring_list_t &errors) {
@@ -643,12 +600,14 @@ bool parenthetical_expression::evaluate(wcstring_list_t &errors) {
 
 // IEEE 1003.1 says nothing about what it means for two strings to be "algebraically equal". For
 // example, should we interpret 0x10 as 0, 10, or 16? Here we use only base 10 and use wcstoll,
-// which allows for leading + and -, and leading whitespace. This matches bash.
-static bool parse_number(const wcstring &arg, long long *out) {
-    const wchar_t *str = arg.c_str();
-    wchar_t *endptr = NULL;
-    *out = wcstoll(str, &endptr, 10);
-    return endptr && *endptr == L'\0';
+// which allows for leading + and -, and whitespace. This is consistent, albeit a bit more lenient
+// since we allow trailing whitespace, with other implementations such as bash.
+static bool parse_number(const wcstring &arg, long long *out, wcstring_list_t &errors) {
+    *out = fish_wcstoll(arg.c_str());
+    if (errno) {
+        errors.push_back(format_string(_(L"invalid integer '%ls'"), arg.c_str()));
+    }
+    return !errno;
 }
 
 static bool binary_primary_evaluate(test_expressions::token_t token, const wcstring &left,
@@ -663,28 +622,28 @@ static bool binary_primary_evaluate(test_expressions::token_t token, const wcstr
             return left != right;
         }
         case test_number_equal: {
-            return parse_number(left, &left_num) && parse_number(right, &right_num) &&
-                   left_num == right_num;
+            return parse_number(left, &left_num, errors) &&
+                   parse_number(right, &right_num, errors) && left_num == right_num;
         }
         case test_number_not_equal: {
-            return parse_number(left, &left_num) && parse_number(right, &right_num) &&
-                   left_num != right_num;
+            return parse_number(left, &left_num, errors) &&
+                   parse_number(right, &right_num, errors) && left_num != right_num;
         }
         case test_number_greater: {
-            return parse_number(left, &left_num) && parse_number(right, &right_num) &&
-                   left_num > right_num;
+            return parse_number(left, &left_num, errors) &&
+                   parse_number(right, &right_num, errors) && left_num > right_num;
         }
         case test_number_greater_equal: {
-            return parse_number(left, &left_num) && parse_number(right, &right_num) &&
-                   left_num >= right_num;
+            return parse_number(left, &left_num, errors) &&
+                   parse_number(right, &right_num, errors) && left_num >= right_num;
         }
         case test_number_lesser: {
-            return parse_number(left, &left_num) && parse_number(right, &right_num) &&
-                   left_num < right_num;
+            return parse_number(left, &left_num, errors) &&
+                   parse_number(right, &right_num, errors) && left_num < right_num;
         }
         case test_number_lesser_equal: {
-            return parse_number(left, &left_num) && parse_number(right, &right_num) &&
-                   left_num <= right_num;
+            return parse_number(left, &left_num, errors) &&
+                   parse_number(right, &right_num, errors) && left_num <= right_num;
         }
         default: {
             errors.push_back(format_string(L"Unknown token type in %s", __func__));
@@ -724,6 +683,13 @@ static bool unary_primary_evaluate(test_expressions::token_t token, const wcstri
         case test_filetype_L: {  // "-L", same as -h
             return !lwstat(arg, &buf) && S_ISLNK(buf.st_mode);
         }
+        case test_filetype_k: {  // "-k", for sticky bit
+#ifdef S_ISVTX
+            return !lwstat(arg, &buf) && buf.st_mode & S_ISVTX;
+#else
+            return false;
+#endif
+        }
         case test_filetype_O: {  // "-O", for check effective user id
             return !wstat(arg, &buf) && geteuid() == buf.st_uid;
         }
@@ -737,7 +703,7 @@ static bool unary_primary_evaluate(test_expressions::token_t token, const wcstri
             return !wstat(arg, &buf) && buf.st_size > 0;
         }
         case test_filedesc_t: {  // "-t", whether the fd is associated with a terminal
-            return parse_number(arg, &num) && num == (int)num && isatty((int)num);
+            return parse_number(arg, &num, errors) && num == (int)num && isatty((int)num);
         }
         case test_fileperm_r: {  // "-r", read permission
             return !waccess(arg, R_OK);
@@ -763,7 +729,7 @@ static bool unary_primary_evaluate(test_expressions::token_t token, const wcstri
         }
     }
 }
-};
+};  // namespace test_expressions
 
 /// Evaluate a conditional expression given the arguments. If fromtest is set, the caller is the
 /// test or [ builtin; with the pointer giving the name of the command. for POSIX conformance this
@@ -775,10 +741,11 @@ int builtin_test(parser_t &parser, io_streams_t &streams, wchar_t **argv) {
     using namespace test_expressions;
 
     // The first argument should be the name of the command ('test').
-    if (!argv[0]) return BUILTIN_TEST_FAIL;
+    if (!argv[0]) return STATUS_INVALID_ARGS;
 
     // Whether we are invoked with bracket '[' or not.
-    const bool is_bracket = !wcscmp(argv[0], L"[");
+    wchar_t *program_name = argv[0];
+    const bool is_bracket = !wcscmp(program_name, L"[");
 
     size_t argc = 0;
     while (argv[argc + 1]) argc++;
@@ -787,53 +754,46 @@ int builtin_test(parser_t &parser, io_streams_t &streams, wchar_t **argv) {
     // of arguments after the command name; thus argv[argc] is the last argument.
     if (is_bracket) {
         if (!wcscmp(argv[argc], L"]")) {
-            // Ignore the closing bracketp.
+            // Ignore the closing bracket from now on.
             argc--;
         } else {
             streams.err.append(L"[: the last argument must be ']'\n");
-            return BUILTIN_TEST_FAIL;
+            return STATUS_INVALID_ARGS;
         }
     }
 
     // Collect the arguments into a list.
     const wcstring_list_t args(argv + 1, argv + 1 + argc);
 
-    switch (argc) {
-        case 0: {
-            // Per 1003.1, exit false.
-            return BUILTIN_TEST_FAIL;
-        }
-        case 1: {
-            // Per 1003.1, exit true if the arg is non-empty.
-            return args.at(0).empty() ? BUILTIN_TEST_FAIL : BUILTIN_TEST_SUCCESS;
-        }
-        default: {
-            // Try parsing. If expr is not nil, we are responsible for deleting it.
-            wcstring err;
-            expression *expr = test_parser::parse_args(args, err);
-            if (!expr) {
-#if 0
-                printf("Oops! test was given args:\n");
-                for (size_t i=0; i < argc; i++) {
-                    printf("\t%ls\n", args.at(i).c_str());
-                }
-                printf("and returned parse error: %ls\n", err.c_str());
-#endif
-                streams.err.append(err);
-                return BUILTIN_TEST_FAIL;
-            }
+    if (argc == 0) {
+        return STATUS_CMD_ERROR;  // Per 1003.1, exit false.
+    } else if (argc == 1) {
+        // Per 1003.1, exit true if the arg is non-empty.
+        return args.at(0).empty() ? STATUS_CMD_ERROR : STATUS_CMD_OK;
+    }
 
-            wcstring_list_t eval_errors;
-            bool result = expr->evaluate(eval_errors);
-            if (!eval_errors.empty()) {
-                printf("test returned eval errors:\n");
-                for (size_t i = 0; i < eval_errors.size(); i++) {
-                    printf("\t%ls\n", eval_errors.at(i).c_str());
-                }
-            }
-            delete expr;
-            return result ? BUILTIN_TEST_SUCCESS : BUILTIN_TEST_FAIL;
+    // Try parsing
+    wcstring err;
+    unique_ptr<expression> expr = test_parser::parse_args(args, err, program_name);
+    if (!expr) {
+#if 0
+        streams.err.append(L"Oops! test was given args:\n");
+        for (size_t i=0; i < argc; i++) {
+            streams.err.append_format(L"\t%ls\n", args.at(i).c_str());
+        }
+        streams.err.append_format(L"and returned parse error: %ls\n", err.c_str());
+#endif
+        streams.err.append(err);
+        return STATUS_CMD_ERROR;
+    }
+
+    wcstring_list_t eval_errors;
+    bool result = expr->evaluate(eval_errors);
+    if (!eval_errors.empty() && !should_suppress_stderr_for_tests()) {
+        streams.err.append(L"test returned eval errors:\n");
+        for (size_t i = 0; i < eval_errors.size(); i++) {
+            streams.err.append_format(L"\t%ls\n", eval_errors.at(i).c_str());
         }
     }
-    return 1;
+    return result ? STATUS_CMD_OK : STATUS_CMD_ERROR;
 }
