@@ -7,6 +7,8 @@
 #define FISH_EVENT_H
 
 #include <unistd.h>
+
+#include <memory>
 #include <vector>
 
 #include "common.h"
@@ -51,8 +53,8 @@ struct event_t {
     /// The type-specific parameter. The int types are one of the following:
     ///
     /// signal: Signal number for signal-type events.Use EVENT_ANY_SIGNAL to match any signal
-    /// pid: Process id for process-type events. Use EVENT_ANY_PID to match any pid.
-    /// job_id: Job id for EVENT_JOB_ID type events
+    /// pid: Process id for process-type events. Use EVENT_ANY_PID to match any pid. (Negative
+    /// values are used for PGIDs). job_id: Job id for EVENT_JOB_ID type events
     union {
         int signal;
         int job_id;
@@ -100,7 +102,7 @@ void event_remove(const event_t &event);
 /// result count will still be valid
 ///
 /// \return the number of found matches
-int event_get(const event_t &criterion, std::vector<event_t *> *out);
+int event_get(const event_t &criterion, std::vector<std::shared_ptr<event_t>> *out);
 
 /// Returns whether an event listener is registered for the given signal. This is safe to call from
 /// a signal handler.
@@ -120,6 +122,7 @@ bool event_is_signal_observed(int signal);
 void event_fire(const event_t *event);
 
 /// Like event_fire, but takes a signal directly.
+/// May be called from signal handlers
 void event_fire_signal(int signal);
 
 /// Initialize the event-handling library.
@@ -128,9 +131,6 @@ void event_init();
 /// Destroy the event-handling library.
 void event_destroy();
 
-/// Free all memory used by the specified event.
-void event_free(event_t *e);
-
 /// Returns a string describing the specified event.
 wcstring event_get_desc(const event_t &e);
 
