@@ -1008,8 +1008,6 @@ void exec_job(parser_t &parser, job_t *j) {
 
                 std::string actual_cmd_str = wcs2string(p->actual_cmd);
                 const char *actual_cmd = actual_cmd_str.c_str();
-
-                const wchar_t *reader_current_filename(void);
                 const wchar_t *file = reader_current_filename();
 
 #if FISH_USE_POSIX_SPAWN
@@ -1064,7 +1062,7 @@ void exec_job(parser_t &parser, job_t *j) {
                         // safe_launch_process _never_ returns...
                         DIE("safe_launch_process should not have returned");
                     } else {
-                        debug(2, L"Fork #%d, pid %d: external command '%s' from '%ls'\n",
+                        debug(2, L"Fork #%d, pid %d: external command '%s' from '%ls'",
                               g_fork_count, pid, p->argv0(), file ? file : L"<no file>");
                         if (pid < 0) {
                             job_mark_process_as_failed(j, p);
@@ -1130,7 +1128,7 @@ void exec_job(parser_t &parser, job_t *j) {
 static int exec_subshell_internal(const wcstring &cmd, wcstring_list_t *lst,
                                   bool apply_exit_status) {
     ASSERT_IS_MAIN_THREAD();
-    int prev_subshell = is_subshell;
+    bool prev_subshell = is_subshell;
     const int prev_status = proc_get_last_status();
     bool split_output = false;
 
@@ -1139,7 +1137,7 @@ static int exec_subshell_internal(const wcstring &cmd, wcstring_list_t *lst,
         split_output = true;
     }
 
-    is_subshell = 1;
+    is_subshell = true;
     int subcommand_status = -1;  // assume the worst
 
     // IO buffer creation may fail (e.g. if we have too many open files to make a pipe), so this may
