@@ -7,7 +7,6 @@
 
 // IWYU likes to recommend adding term.h when we want ncurses.h.
 // IWYU pragma: no_include term.h
-#include <assert.h>  // IWYU pragma: keep
 #include <dirent.h>  // IWYU pragma: keep
 #include <errno.h>   // IWYU pragma: keep
 #include <fcntl.h>   // IWYU pragma: keep
@@ -91,10 +90,24 @@ char *tparm_solaris_kludge(char *str, ...) {
 
 #endif
 
-#if __APPLE__
+int fish_mkstemp_cloexec(char *name_template) {
+#if HAVE_MKOSTEMP
+    // null check because mkostemp may be a weak symbol
+    if (&mkostemp != nullptr) {
+        return mkostemp(name_template, O_CLOEXEC);
+    }
+#endif
+    int result_fd = mkstemp(name_template);
+    if (result_fd != -1) {
+        fcntl(result_fd, F_SETFD, FD_CLOEXEC);
+    }
+    return result_fd;
+}
+
 /// Fallback implementations of wcsdup and wcscasecmp. On systems where these are not needed (e.g.
 /// building on Linux) these should end up just being stripped, as they are static functions that
 /// are not referenced in this file.
+// cppcheck-suppress unusedFunction
 __attribute__((unused)) static wchar_t *wcsdup_fallback(const wchar_t *in) {
     size_t len = wcslen(in);
     wchar_t *out = (wchar_t *)malloc(sizeof(wchar_t) * (len + 1));
@@ -105,7 +118,6 @@ __attribute__((unused)) static wchar_t *wcsdup_fallback(const wchar_t *in) {
     memcpy(out, in, sizeof(wchar_t) * (len + 1));
     return out;
 }
-#endif
 
 __attribute__((unused)) static int wcscasecmp_fallback(const wchar_t *a, const wchar_t *b) {
     if (*a == 0) {
@@ -158,6 +170,28 @@ int wcsncasecmp(const wchar_t *a, const wchar_t *b, size_t n) {
     return wcsncasecmp_fallback(a, b, n);
 }
 #endif  // __DARWIN_C_LEVEL >= 200809L
+#else   // __APPLE__
+
+#ifndef HAVE_WCSDUP
+#ifndef HAVE_STD__WCSDUP
+wchar_t *wcsdup(const wchar_t *in) { return wcsdup_fallback(in); }
+#endif
+#endif
+
+#ifndef HAVE_WCSCASECMP
+#ifndef HAVE_STD__WCSCASECMP
+int wcscasecmp(const wchar_t *a, const wchar_t *b) { return wcscasecmp_fallback(a, b); }
+#endif
+#endif
+
+#ifndef HAVE_WCSNCASECMP
+#ifndef HAVE_STD__WCSNCASECMP
+int wcsncasecmp(const wchar_t *a, const wchar_t *b, size_t n) {
+    return wcsncasecmp_fallback(a, b, n);
+}
+#endif
+#endif
+
 #endif  // __APPLE__
 
 #ifndef HAVE_WCSNDUP
@@ -190,9 +224,9 @@ wchar_t *wcsndup(const wchar_t *in, size_t c) {
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 size_t wcslcpy(wchar_t *dst, const wchar_t *src, size_t siz) {
-    register wchar_t *d = dst;
-    register const wchar_t *s = src;
-    register size_t n = siz;
+    wchar_t *d = dst;
+    const wchar_t *s = src;
+    size_t n = siz;
 
     // Copy as many bytes as will fit.
     if (n != 0 && --n != 0) {
@@ -203,16 +237,16 @@ size_t wcslcpy(wchar_t *dst, const wchar_t *src, size_t siz) {
 
     // Not enough room in dst, add NUL and traverse rest of src.
     if (n == 0) {
-        if (siz != 0) *d = '\0';
-        // NUL-terminate dst.
+        if (siz != 0) *d = '\0';  // NUL-terminate dst
         while (*s++)
-            ;
+            ;  // ignore rest of src
     }
-    return s - src - 1;
-    // Count does not include NUL.
+    return s - src - 1;  // count does not include NUL
 }
 #endif
 
+#if 0
+// These are not currently used.
 #ifndef HAVE_LRAND48_R
 int lrand48_r(struct drand48_data *buffer, long int *result) {
     *result = rand_r(&buffer->seed);
@@ -224,6 +258,7 @@ int srand48_r(long int seedval, struct drand48_data *buffer) {
     return 0;
 }
 #endif
+#endif
 
 #ifndef HAVE_FUTIMES
 int futimes(int fd, const struct timeval *times) {
@@ -353,7 +388,7 @@ static int bisearch(wchar_t ucs, const struct interval *table, int max) {
         if (ucs > table[mid].last)
             min = mid + 1;
         else if (ucs < table[mid].first)
-            max = mid - 1;
+            max = mid - 1;  //!OCLINT(parameter reassignment)
         else
             return 1;
     }
@@ -467,3 +502,72 @@ static int mk_wcswidth(const wchar_t *pwcs, size_t n) {
     return width;
 }
 #endif  // HAVE_BROKEN_WCWIDTH
+
+#ifndef HAVE_FLOCK
+/*	$NetBSD: flock.c,v 1.6 2008/04/28 20:24:12 martin Exp $	*/
+
+/*-
+ * Copyright (c) 2001 The NetBSD Foundation, Inc.
+ * All rights reserved.
+ *
+ * This code is derived from software contributed to The NetBSD Foundation
+ * by Todd Vierling.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
+ * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+ * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ */
+
+/*
+ * Emulate flock() with fcntl().
+ */
+
+int flock(int fd, int op) {
+    int rc = 0;
+
+    struct flock fl = {0};
+
+    switch (op & (LOCK_EX | LOCK_SH | LOCK_UN)) {
+        case LOCK_EX:
+            fl.l_type = F_WRLCK;
+            break;
+
+        case LOCK_SH:
+            fl.l_type = F_RDLCK;
+            break;
+
+        case LOCK_UN:
+            fl.l_type = F_UNLCK;
+            break;
+
+        default:
+            errno = EINVAL;
+            return -1;
+    }
+
+    fl.l_whence = SEEK_SET;
+    rc = fcntl(fd, op & LOCK_NB ? F_SETLK : F_SETLKW, &fl);
+
+    if (rc && (errno == EAGAIN)) errno = EWOULDBLOCK;
+
+    return rc;
+}
+
+#endif  // HAVE_FLOCK
