@@ -4,16 +4,16 @@
 // IWYU pragma: no_include <cstddef>
 #include <dirent.h>
 #include <errno.h>
-#include <limits.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <wchar.h>
-#include <wctype.h>
+
 #include <algorithm>
 #include <map>
 #include <memory>
 #include <set>
 #include <string>
+#include <type_traits>
 #include <utility>
 
 #include "builtin.h"
@@ -41,22 +41,35 @@
 
 /// The environment variables used to specify the color of different tokens. This matches the order
 /// in highlight_spec_t.
-static const wchar_t *const highlight_var[] = {
-    L"fish_color_normal", L"fish_color_error", L"fish_color_command", L"fish_color_end",
-    L"fish_color_param", L"fish_color_comment", L"fish_color_match", L"fish_color_search_match",
-    L"fish_color_operator", L"fish_color_escape", L"fish_color_quote", L"fish_color_redirection",
-    L"fish_color_autosuggestion", L"fish_color_selection",
+static const wchar_t *const highlight_var[] = {L"fish_color_normal",
+                                               L"fish_color_error",
+                                               L"fish_color_command",
+                                               L"fish_color_end",
+                                               L"fish_color_param",
+                                               L"fish_color_comment",
+                                               L"fish_color_match",
+                                               L"fish_color_search_match",
+                                               L"fish_color_operator",
+                                               L"fish_color_escape",
+                                               L"fish_color_quote",
+                                               L"fish_color_redirection",
+                                               L"fish_color_autosuggestion",
+                                               L"fish_color_selection",
+                                               L"fish_pager_color_prefix",
+                                               L"fish_pager_color_completion",
+                                               L"fish_pager_color_description",
+                                               L"fish_pager_color_progress",
+                                               L"fish_pager_color_secondary"};
 
-    L"fish_pager_color_prefix", L"fish_pager_color_completion", L"fish_pager_color_description",
-    L"fish_pager_color_progress", L"fish_pager_color_secondary"
-
-};
-
-/// Determine if the filesystem containing the given fd is case insensitive.
+/// Determine if the filesystem containing the given fd is case insensitive for lookups regardless
+/// of whether it preserves the case when saving a pathname.
+///
+/// Returns:
+///     false: the filesystem is not case insensitive
+///     true: the file system is case insensitive
 typedef std::map<wcstring, bool> case_sensitivity_cache_t;
 bool fs_is_case_insensitive(const wcstring &path, int fd,
                             case_sensitivity_cache_t &case_sensitivity_cache) {
-    // If _PC_CASE_SENSITIVE is not defined, assume case sensitive.
     bool result = false;
 #ifdef _PC_CASE_SENSITIVE
     // Try the cache first.
@@ -71,6 +84,11 @@ bool fs_is_case_insensitive(const wcstring &path, int fd,
         result = (ret == 0);
         case_sensitivity_cache[path] = result;
     }
+#else
+    // Silence lint tools about the unused parameters.
+    UNUSED(path);
+    UNUSED(fd);
+    UNUSED(case_sensitivity_cache);
 #endif
     return result;
 }
@@ -87,7 +105,7 @@ bool is_potential_path(const wcstring &potential_path_fragment, const wcstring_l
                        path_flags_t flags) {
     ASSERT_IS_BACKGROUND_THREAD();
 
-    const bool require_dir = !!(flags & PATH_REQUIRE_DIR);
+    const bool require_dir = static_cast<bool>(flags & PATH_REQUIRE_DIR);
     wcstring clean_potential_path_fragment;
     int has_magic = 0;
     bool result = false;
@@ -122,73 +140,73 @@ bool is_potential_path(const wcstring &potential_path_fragment, const wcstring_l
         }
     }
 
-    if (!has_magic && !clean_potential_path_fragment.empty()) {
-        // Don't test the same path multiple times, which can happen if the path is absolute and the
-        // CDPATH contains multiple entries.
-        std::set<wcstring> checked_paths;
+    if (has_magic || clean_potential_path_fragment.empty()) {
+        return result;
+    }
 
-        // Keep a cache of which paths / filesystems are case sensitive.
-        case_sensitivity_cache_t case_sensitivity_cache;
+    // Don't test the same path multiple times, which can happen if the path is absolute and the
+    // CDPATH contains multiple entries.
+    std::set<wcstring> checked_paths;
 
-        for (size_t wd_idx = 0; wd_idx < directories.size() && !result; wd_idx++) {
-            const wcstring &wd = directories.at(wd_idx);
+    // Keep a cache of which paths / filesystems are case sensitive.
+    case_sensitivity_cache_t case_sensitivity_cache;
 
-            const wcstring abs_path =
-                path_apply_working_directory(clean_potential_path_fragment, wd);
+    for (size_t wd_idx = 0; wd_idx < directories.size() && !result; wd_idx++) {
+        const wcstring &wd = directories.at(wd_idx);
 
-            // Skip this if it's empty or we've already checked it.
-            if (abs_path.empty() || checked_paths.count(abs_path)) continue;
-            checked_paths.insert(abs_path);
+        const wcstring abs_path = path_apply_working_directory(clean_potential_path_fragment, wd);
 
-            // If we end with a slash, then it must be a directory.
-            bool must_be_full_dir = abs_path.at(abs_path.size() - 1) == L'/';
-            if (must_be_full_dir) {
-                struct stat buf;
-                if (0 == wstat(abs_path, &buf) && S_ISDIR(buf.st_mode)) {
-                    result = true;
-                }
-            } else {
-                // We do not end with a slash; it does not have to be a directory.
-                DIR *dir = NULL;
-                const wcstring dir_name = wdirname(abs_path);
-                const wcstring filename_fragment = wbasename(abs_path);
-                if (dir_name == L"/" && filename_fragment == L"/") {
-                    // cd ///.... No autosuggestion.
-                    result = true;
-                } else if ((dir = wopendir(dir_name))) {
-                    // Check if we're case insensitive.
-                    const bool do_case_insensitive =
-                        fs_is_case_insensitive(dir_name, dirfd(dir), case_sensitivity_cache);
+        // Skip this if it's empty or we've already checked it.
+        if (abs_path.empty() || checked_paths.count(abs_path)) continue;
+        checked_paths.insert(abs_path);
 
-                    wcstring matched_file;
+        // If we end with a slash, then it must be a directory.
+        bool must_be_full_dir = abs_path.at(abs_path.size() - 1) == L'/';
+        if (must_be_full_dir) {
+            struct stat buf;
+            if (0 == wstat(abs_path, &buf) && S_ISDIR(buf.st_mode)) {
+                result = true;
+            }
+        } else {
+            // We do not end with a slash; it does not have to be a directory.
+            DIR *dir = NULL;
+            const wcstring dir_name = wdirname(abs_path);
+            const wcstring filename_fragment = wbasename(abs_path);
+            if (dir_name == L"/" && filename_fragment == L"/") {
+                // cd ///.... No autosuggestion.
+                result = true;
+            } else if ((dir = wopendir(dir_name))) {
+                // Check if we're case insensitive.
+                const bool do_case_insensitive =
+                    fs_is_case_insensitive(dir_name, dirfd(dir), case_sensitivity_cache);
 
-                    // We opened the dir_name; look for a string where the base name prefixes it
-                    // Don't ask for the is_dir value unless we care, because it can cause extra
-                    // filesystem access.
-                    wcstring ent;
-                    bool is_dir = false;
-                    while (wreaddir_resolving(dir, dir_name, ent, require_dir ? &is_dir : NULL)) {
-                        // Maybe skip directories.
-                        if (require_dir && !is_dir) {
-                            continue;
-                        }
+                wcstring matched_file;
 
-                        if (string_prefixes_string(filename_fragment, ent) ||
-                            (do_case_insensitive &&
-                             string_prefixes_string_case_insensitive(filename_fragment, ent))) {
-                            // We matched.
-                            matched_file = ent;
-                            break;
-                        }
+                // We opened the dir_name; look for a string where the base name prefixes it Don't
+                // ask for the is_dir value unless we care, because it can cause extra filesystem
+                // access.
+                wcstring ent;
+                bool is_dir = false;
+                while (wreaddir_resolving(dir, dir_name, ent, require_dir ? &is_dir : NULL)) {
+                    // Maybe skip directories.
+                    if (require_dir && !is_dir) {
+                        continue;
                     }
-                    closedir(dir);
 
-                    // We succeeded if we found a match.
-                    result = !matched_file.empty();
+                    if (string_prefixes_string(filename_fragment, ent) ||
+                        (do_case_insensitive &&
+                         string_prefixes_string_case_insensitive(filename_fragment, ent))) {
+                        matched_file = ent;  // we matched
+                        break;
+                    }
                 }
+                closedir(dir);
+
+                result = !matched_file.empty();  // we succeeded if we found a match
             }
         }
     }
+
     return result;
 }
 
@@ -207,9 +225,10 @@ static bool is_potential_cd_path(const wcstring &path, const wcstring &working_d
         if (cdpath.missing_or_empty()) cdpath = L".";
 
         // Tokenize it into directories.
-        wcstokenizer tokenizer(cdpath, ARRAY_SEP_STR);
-        wcstring next_path;
-        while (tokenizer.next(next_path)) {
+        std::vector<wcstring> pathsv;
+        tokenize_variable_array(cdpath, pathsv);
+        for (auto next_path : pathsv) {
+            if (next_path.empty()) next_path = L".";
             // Ensure that we use the working directory for relative cdpaths like ".".
             directories.push_back(path_apply_working_directory(next_path, working_directory));
         }
@@ -224,19 +243,16 @@ static bool is_potential_cd_path(const wcstring &path, const wcstring &working_d
 bool plain_statement_get_expanded_command(const wcstring &src, const parse_node_tree_t &tree,
                                           const parse_node_t &plain_statement, wcstring *out_cmd) {
     assert(plain_statement.type == symbol_plain_statement);
-    bool result = false;
 
-    // Get the command.
+    // Get the command. Try expanding it. If we cannot, it's an error.
     wcstring cmd;
-    if (tree.command_for_plain_statement(plain_statement, src, &cmd)) {
-        // Try expanding it. If we cannot, it's an error.
-        if (expand_one(cmd, EXPAND_SKIP_CMDSUBST | EXPAND_SKIP_VARIABLES | EXPAND_SKIP_JOBS)) {
-            // Success, return the expanded string by reference.
-            out_cmd->swap(cmd);
-            result = true;
-        }
+    if (tree.command_for_plain_statement(plain_statement, src, &cmd) &&
+        expand_one(cmd, EXPAND_SKIP_CMDSUBST | EXPAND_SKIP_VARIABLES | EXPAND_SKIP_JOBS)) {
+        // Success, return the expanded string by reference.
+        *out_cmd = std::move(cmd);
+        return true;
     }
-    return result;
+    return false;
 }
 
 rgb_color_t highlight_get_color(highlight_spec_t highlight, bool is_background) {
@@ -271,6 +287,9 @@ rgb_color_t highlight_get_color(highlight_spec_t highlight, bool is_background)
         else {
             if (result2.is_bold()) result.set_bold(true);
             if (result2.is_underline()) result.set_underline(true);
+            if (result2.is_italics()) result.set_italics(true);
+            if (result2.is_dim()) result.set_dim(true);
+            if (result2.is_reverse()) result.set_reverse(true);
         }
     }
 
@@ -297,8 +316,6 @@ static bool has_expand_reserved(const wcstring &str) {
 // (as a copied node), if any. This is used by autosuggestions.
 static bool autosuggest_parse_command(const wcstring &buff, wcstring *out_expanded_command,
                                       parse_node_t *out_last_arg) {
-    bool result = false;
-
     // Parse the buffer.
     parse_node_tree_t parse_tree;
     parse_tree_from_string(buff,
@@ -308,25 +325,20 @@ static bool autosuggest_parse_command(const wcstring &buff, wcstring *out_expand
     // Find the last statement.
     const parse_node_t *last_statement =
         parse_tree.find_last_node_of_type(symbol_plain_statement, NULL);
-    if (last_statement != NULL) {
-        if (plain_statement_get_expanded_command(buff, parse_tree, *last_statement,
-                                                 out_expanded_command)) {
-            // We got it.
-            result = true;
-
-            // Find the last argument. If we don't get one, return an invalid node.
-            const parse_node_t *last_arg =
-                parse_tree.find_last_node_of_type(symbol_argument, last_statement);
-            if (last_arg != NULL) {
-                *out_last_arg = *last_arg;
-            }
+    if (last_statement != NULL && plain_statement_get_expanded_command(
+                                      buff, parse_tree, *last_statement, out_expanded_command)) {
+        // Find the last argument. If we don't get one, return an invalid node.
+        const parse_node_t *last_arg =
+            parse_tree.find_last_node_of_type(symbol_argument, last_statement);
+        if (last_arg != NULL) {
+            *out_last_arg = *last_arg;
         }
+        return true;
     }
-    return result;
+    return false;
 }
 
 bool autosuggest_validate_from_history(const history_item_t &item,
-                                       file_detection_context_t &detector,
                                        const wcstring &working_directory,
                                        const env_vars_snapshot_t &vars) {
     ASSERT_IS_BACKGROUND_THREAD();
@@ -346,42 +358,32 @@ bool autosuggest_validate_from_history(const history_item_t &item,
             handled = true;
             bool is_help =
                 string_prefixes_string(dir, L"--help") || string_prefixes_string(dir, L"-h");
-            if (is_help) {
-                suggestionOK = false;
-            } else {
+            if (!is_help) {
                 wcstring path;
                 bool can_cd = path_get_cdpath(dir, &path, working_directory.c_str(), vars);
-                if (!can_cd) {
-                    suggestionOK = false;
-                } else if (paths_are_same_file(working_directory, path)) {
-                    // Don't suggest the working directory as the path!
-                    suggestionOK = false;
-                } else {
+                if (can_cd && !paths_are_same_file(working_directory, path)) {
                     suggestionOK = true;
                 }
             }
         }
     }
 
-    // If not handled specially, handle it here.
-    if (!handled) {
-        bool cmd_ok = false;
+    if (handled) {
+        return suggestionOK;
+    }
 
-        if (path_get_path(parsed_command, NULL)) {
-            cmd_ok = true;
-        } else if (builtin_exists(parsed_command) ||
-                   function_exists_no_autoload(parsed_command, vars)) {
-            cmd_ok = true;
-        }
+    // Not handled specially so handle it here.
+    bool cmd_ok = false;
+    if (path_get_path(parsed_command, NULL)) {
+        cmd_ok = true;
+    } else if (builtin_exists(parsed_command) ||
+               function_exists_no_autoload(parsed_command, vars)) {
+        cmd_ok = true;
+    }
 
-        if (cmd_ok) {
-            const path_list_t &paths = item.get_required_paths();
-            if (paths.empty()) {
-                suggestionOK = true;
-            } else {
-                suggestionOK = detector.paths_are_valid(paths);
-            }
-        }
+    if (cmd_ok) {
+        const path_list_t &paths = item.get_required_paths();
+        suggestionOK = all_paths_are_valid(paths, working_directory);
     }
 
     return suggestionOK;
@@ -400,7 +402,7 @@ static size_t color_variable(const wchar_t *in, size_t in_len,
     while (in[idx] == '$') {
         // Our color depends on the next char.
         wchar_t next = in[idx + 1];
-        if (next == L'$' || wcsvarchr(next)) {
+        if (next == L'$' || valid_var_name_char(next)) {
             colors[idx] = highlight_spec_operator;
         } else {
             colors[idx] = highlight_spec_error;
@@ -410,7 +412,7 @@ static size_t color_variable(const wchar_t *in, size_t in_len,
     }
 
     // Handle a sequence of variable characters.
-    while (wcsvarchr(in[idx])) {
+    while (valid_var_name_char(in[idx])) {
         colors[idx++] = highlight_spec_operator;
     }
 
@@ -587,6 +589,9 @@ static void color_argument_internal(const wcstring &buffstr,
                             mode = e_double_quoted;
                             break;
                         }
+                        default: {
+                            break;  // we ignore all other characters
+                        }
                     }
                 }
                 break;
@@ -640,6 +645,9 @@ static void color_argument_internal(const wcstring &buffstr,
                         in_pos -= 1;
                         break;
                     }
+                    default: {
+                        break;  // we ignore all other characters
+                    }
                 }
                 break;
             }
@@ -844,9 +852,8 @@ void highlighter_t::color_redirection(const parse_node_t &redirection_node) {
             this->parse_tree.type_for_redirection(redirection_node, this->buff, NULL, &target);
 
         // We may get a TOK_NONE redirection type, e.g. if the redirection is invalid.
-        this->color_node(*redirection_primitive, redirect_type == TOK_NONE
-                                                     ? highlight_spec_error
-                                                     : highlight_spec_redirection);
+        auto hl = redirect_type == TOK_NONE ? highlight_spec_error : highlight_spec_redirection;
+        this->color_node(*redirection_primitive, hl);
 
         // Check if the argument contains a command substitution. If so, highlight it as a param
         // even though it's a command redirection, and don't try to do any other validation.
@@ -874,19 +881,8 @@ void highlighter_t::color_redirection(const parse_node_t &redirection_node) {
                     path_apply_working_directory(target, this->working_directory);
                 switch (redirect_type) {
                     case TOK_REDIRECT_FD: {
-                        // Target should be an fd. It must be all digits, and must not overflow.
-                        // fish_wcstoi returns INT_MAX on overflow; we could instead check errno to
-                        // disambiguiate this from a real INT_MAX fd, but instead we just disallow
-                        // that.
-                        const wchar_t *target_cstr = target.c_str();
-                        wchar_t *end = NULL;
-                        int fd = fish_wcstoi(target_cstr, &end, 10);
-
-                        // The iswdigit check ensures there's no leading whitespace, the *end check
-                        // ensures the entire string was consumed, and the numeric checks ensure the
-                        // fd is at least zero and there was no overflow.
-                        target_is_valid =
-                            (iswdigit(target_cstr[0]) && *end == L'\0' && fd >= 0 && fd < INT_MAX);
+                        int fd = fish_wcstoi(target.c_str());
+                        target_is_valid = !errno && fd >= 0;
                         break;
                     }
                     case TOK_REDIRECT_IN: {
@@ -954,8 +950,8 @@ void highlighter_t::color_redirection(const parse_node_t &redirection_node) {
             }
 
             if (redirection_target != NULL) {
-                this->color_node(*redirection_target, target_is_valid ? highlight_spec_redirection
-                                                                      : highlight_spec_error);
+                auto hl = target_is_valid ? highlight_spec_redirection : highlight_spec_error;
+                this->color_node(*redirection_target, hl);
             }
         }
     }
@@ -1041,7 +1037,7 @@ const highlighter_t::color_array_t &highlighter_t::highlight() {
 #if 0
     // Disabled for the 2.2.0 release: https://github.com/fish-shell/fish-shell/issues/1809.
     const wcstring dump = parse_dump_tree(parse_tree, buff);
-    fprintf(stderr, "%ls\n", dump.c_str());
+    fwprintf(stderr, L"%ls\n", dump.c_str());
 #endif
 
     // Walk the node tree.
@@ -1102,26 +1098,27 @@ const highlighter_t::color_array_t &highlighter_t::highlight() {
                 // Color the command.
                 const parse_node_t *cmd_node =
                     parse_tree.get_child(node, 0, parse_token_type_string);
-                if (cmd_node != NULL && cmd_node->has_source()) {
-                    bool is_valid_cmd = false;
-                    if (!this->io_ok) {
-                        // We cannot check if the command is invalid, so just assume it's valid.
-                        is_valid_cmd = true;
-                    } else {
-                        // Check to see if the command is valid.
-                        wcstring cmd(buff, cmd_node->source_start, cmd_node->source_length);
-
-                        // Try expanding it. If we cannot, it's an error.
-                        bool expanded = expand_one(
-                            cmd, EXPAND_SKIP_CMDSUBST | EXPAND_SKIP_VARIABLES | EXPAND_SKIP_JOBS);
-                        if (expanded && !has_expand_reserved(cmd)) {
-                            is_valid_cmd =
-                                command_is_valid(cmd, decoration, working_directory, vars);
-                        }
-                    }
-                    this->color_node(*cmd_node,
-                                     is_valid_cmd ? highlight_spec_command : highlight_spec_error);
+                if (cmd_node == NULL || !cmd_node->has_source()) {
+                    break;  // not much as we can do without a node that has source text
                 }
+
+                bool is_valid_cmd = false;
+                if (!this->io_ok) {
+                    // We cannot check if the command is invalid, so just assume it's valid.
+                    is_valid_cmd = true;
+                } else {
+                    // Check to see if the command is valid.
+                    wcstring cmd(buff, cmd_node->source_start, cmd_node->source_length);
+
+                    // Try expanding it. If we cannot, it's an error.
+                    bool expanded = expand_one(
+                        cmd, EXPAND_SKIP_CMDSUBST | EXPAND_SKIP_VARIABLES | EXPAND_SKIP_JOBS);
+                    if (expanded && !has_expand_reserved(cmd)) {
+                        is_valid_cmd = command_is_valid(cmd, decoration, working_directory, vars);
+                    }
+                }
+                this->color_node(*cmd_node,
+                                 is_valid_cmd ? highlight_spec_command : highlight_spec_error);
                 break;
             }
             case symbol_arguments_or_redirections_list:
@@ -1150,32 +1147,29 @@ const highlighter_t::color_array_t &highlighter_t::highlight() {
         }
     }
 
-    if (this->io_ok && this->cursor_pos <= this->buff.size()) {
-        // If the cursor is over an argument, and that argument is a valid path, underline it.
-        for (parse_node_tree_t::const_iterator iter = parse_tree.begin(); iter != parse_tree.end();
-             ++iter) {
-            const parse_node_t &node = *iter;
+    if (!this->io_ok || this->cursor_pos > this->buff.size()) {
+        return color_array;
+    }
 
-            // Must be an argument with source.
-            if (node.type != symbol_argument || !node.has_source()) continue;
+    // If the cursor is over an argument, and that argument is a valid path, underline it.
+    for (parse_node_tree_t::const_iterator iter = parse_tree.begin(); iter != parse_tree.end();
+         ++iter) {
+        const parse_node_t &node = *iter;
 
-            // See if this node contains the cursor. We check <= source_length so that, when
-            // backspacing (and the cursor is just beyond the last token), we may still underline
-            // it.
-            if (this->cursor_pos >= node.source_start &&
-                this->cursor_pos - node.source_start <= node.source_length) {
-                // See if this is a valid path.
-                if (node_is_potential_path(buff, node, working_directory)) {
-                    // It is, underline it.
-                    for (size_t i = node.source_start; i < node.source_start + node.source_length;
-                         i++) {
-                        // Don't color highlight_spec_error because it looks dorky. For example,
-                        // trying to cd into a non-directory would show an underline and also red.
-                        if (highlight_get_primary(this->color_array.at(i)) !=
-                            highlight_spec_error) {
-                            this->color_array.at(i) |= highlight_modifier_valid_path;
-                        }
-                    }
+        // Must be an argument with source.
+        if (node.type != symbol_argument || !node.has_source()) continue;
+
+        // See if this node contains the cursor. We check <= source_length so that, when backspacing
+        // (and the cursor is just beyond the last token), we may still underline it.
+        if (this->cursor_pos >= node.source_start &&
+            this->cursor_pos - node.source_start <= node.source_length &&
+            node_is_potential_path(buff, node, working_directory)) {
+            // It is, underline it.
+            for (size_t i = node.source_start; i < node.source_start + node.source_length; i++) {
+                // Don't color highlight_spec_error because it looks dorky. For example,
+                // trying to cd into a non-directory would show an underline and also red.
+                if (highlight_get_primary(this->color_array.at(i)) != highlight_spec_error) {
+                    this->color_array.at(i) |= highlight_modifier_valid_path;
                 }
             }
         }
@@ -1257,6 +1251,9 @@ static void highlight_universal_internal(const wcstring &buffstr,
                         }
                         break;
                     }
+                    default: {
+                        break;  // we ignore all other characters
+                    }
                 }
                 if ((*str == L'\0')) break;
                 str++;
