@@ -52,6 +52,9 @@
 //
 //   Newlines are replaced by \n. Backslashes are replaced by \\.
 
+// This is the history session ID we use by default if the user has not set env var fish_history.
+#define DFLT_FISH_HISTORY_SESSION_ID L"fish"
+
 // When we rewrite the history, the number of items we keep.
 #define HISTORY_SAVE_MAX (1024 * 256)
 
@@ -1139,13 +1142,14 @@ static void unescape_yaml(std::string *str) {
     }
 }
 
-static wcstring history_filename(const wcstring &name, const wcstring &suffix) {
-    wcstring path;
-    if (!path_get_data(path)) return L"";
+static wcstring history_filename(const wcstring &session_id, const wcstring &suffix) {
+    if (session_id.empty()) return L"";
+
+    wcstring result;
+    if (!path_get_data(result)) return L"";
 
-    wcstring result = path;
     result.append(L"/");
-    result.append(name);
+    result.append(session_id);
     result.append(L"_history");
     result.append(suffix);
     return result;
@@ -1401,6 +1405,9 @@ bool history_t::save_internal_via_appending() {
 
     // Get the path to the real history file.
     wcstring history_path = history_filename(name, wcstring());
+    if (history_path.empty()) {
+        return true;
+    }
 
     signal_block();
 
@@ -1508,6 +1515,13 @@ void history_t::save_internal(bool vacuum) {
     // Nothing to do if there's no new items.
     if (first_unwritten_new_item_index >= new_items.size() && deleted_items.empty()) return;
 
+    if (history_filename(name, L"").empty()) {
+        // We're in the "incognito" mode. Pretend we've saved the history.
+        this->first_unwritten_new_item_index = new_items.size();
+        this->deleted_items.clear();
+        this->clear_file_state();
+    }
+
     // Compact our new items so we don't have duplicates.
     this->compact_new_items();
 
@@ -1627,6 +1641,10 @@ bool history_t::is_empty(void) {
         // If we have not loaded old items, don't actually load them (which may be expensive); just
         // stat the file and see if it exists and is nonempty.
         const wcstring where = history_filename(name, L"");
+        if (where.empty()) {
+            return true;
+        }
+
         struct stat buf = {};
         if (wstat(where, &buf) != 0) {
             // Access failed, assume missing.
@@ -1643,6 +1661,11 @@ bool history_t::is_empty(void) {
 /// clearing ourselves, and copying the contents of the old history file to the new history file.
 /// The new contents will automatically be re-mapped later.
 void history_t::populate_from_config_path() {
+    wcstring new_file = history_filename(name, wcstring());
+    if (new_file.empty()) {
+        return;
+    }
+
     wcstring old_file;
     if (path_get_config(old_file)) {
         old_file.append(L"/");
@@ -1650,8 +1673,6 @@ void history_t::populate_from_config_path() {
         old_file.append(L"_history");
         int src_fd = wopen_cloexec(old_file, O_RDONLY, 0);
         if (src_fd != -1) {
-            wcstring new_file = history_filename(name, wcstring());
-
             // Clear must come after we've retrieved the new_file name, and before we open
             // destination file descriptor, since it destroys the name and the file.
             this->clear();
@@ -1712,9 +1733,11 @@ static bool should_import_bash_history_line(const std::string &line) {
 /// commands. We can't actually parse bash syntax and the bash history file does not unambiguously
 /// encode multiline commands.
 void history_t::populate_from_bash(FILE *stream) {
-    bool eof = false;
+    // We do not import bash history if an alternative fish history file is being used.
+    if (history_session_id() != DFLT_FISH_HISTORY_SESSION_ID) return;
 
     // Process the entire history file until EOF is observed.
+    bool eof = false;
     while (!eof) {
         auto line = std::string();
 
@@ -1778,6 +1801,29 @@ void history_sanity_check() {
     // No sanity checking implemented yet...
 }
 
+/// Return the prefix for the files to be used for command and read history.
+wcstring history_session_id() {
+    wcstring result = DFLT_FISH_HISTORY_SESSION_ID;
+
+    const env_var_t session_id = env_get_string(L"fish_history");
+    if (!session_id.missing()) {
+        if (session_id.empty()) {
+            result = L"";
+        } else if (session_id == L"default") {
+            ;  // using the default value
+        } else if (valid_var_name(session_id)) {
+            result = session_id;
+        } else {
+            debug(0,
+                  _(L"History session ID '%ls' is not a valid variable name. "
+                    L"Falling back to `%ls`."),
+                  session_id.c_str(), result.c_str());
+        }
+    }
+
+    return result;
+}
+
 path_list_t valid_paths(const path_list_t &paths, const wcstring &working_directory) {
     ASSERT_IS_BACKGROUND_THREAD();
     wcstring_list_t result;
