@@ -1,11 +1,11 @@
 // Utilities for io redirection.
 #include "config.h"  // IWYU pragma: keep
 
-#include <assert.h>
 #include <errno.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <unistd.h>
+#include <wchar.h>
 
 #include "common.h"
 #include "exec.h"
@@ -15,19 +15,20 @@
 
 io_data_t::~io_data_t() {}
 
-void io_close_t::print() const { fprintf(stderr, "close %d\n", fd); }
+void io_close_t::print() const { fwprintf(stderr, L"close %d\n", fd); }
 
-void io_fd_t::print() const { fprintf(stderr, "FD map %d -> %d\n", old_fd, fd); }
+void io_fd_t::print() const { fwprintf(stderr, L"FD map %d -> %d\n", old_fd, fd); }
 
-void io_file_t::print() const { fprintf(stderr, "file (%s)\n", filename_cstr); }
+void io_file_t::print() const { fwprintf(stderr, L"file (%s)\n", filename_cstr); }
 
 void io_pipe_t::print() const {
-    fprintf(stderr, "pipe {%d, %d} (input: %s)\n", pipe_fd[0], pipe_fd[1], is_input ? "yes" : "no");
+    fwprintf(stderr, L"pipe {%d, %d} (input: %s)\n", pipe_fd[0], pipe_fd[1],
+             is_input ? "yes" : "no");
 }
 
 void io_buffer_t::print() const {
-    fprintf(stderr, "buffer %p (input: %s, size %lu)\n", out_buffer_ptr(), is_input ? "yes" : "no",
-            (unsigned long)out_buffer_size());
+    fwprintf(stderr, L"buffer %p (input: %s, size %lu)\n", out_buffer_ptr(),
+             is_input ? "yes" : "no", (unsigned long)out_buffer_size());
 }
 
 void io_buffer_t::read() {
@@ -52,9 +53,9 @@ void io_buffer_t::read() {
                 // never block. But a broken pipe seems to cause some flags to reset, causing the
                 // EOF flag to not be set. Therefore, EAGAIN is ignored and we exit anyway.
                 if (errno != EAGAIN) {
-                    debug(1, _(L"An error occured while reading output from code block on file "
-                               L"descriptor %d"),
-                          pipe_fd[0]);
+                    const wchar_t *fmt =
+                        _(L"An error occured while reading output from code block on fd %d");
+                    debug(1, fmt, pipe_fd[0]);
                     wperror(L"io_buffer_t::read");
                 }
 
@@ -74,10 +75,10 @@ bool io_buffer_t::avoid_conflicts_with_io_chain(const io_chain_t &ios) {
     return result;
 }
 
-io_buffer_t *io_buffer_t::create(int fd, const io_chain_t &conflicts) {
+shared_ptr<io_buffer_t> io_buffer_t::create(int fd, const io_chain_t &conflicts) {
     bool success = true;
     assert(fd >= 0);
-    io_buffer_t *buffer_redirect = new io_buffer_t(fd);
+    shared_ptr<io_buffer_t> buffer_redirect(new io_buffer_t(fd));
 
     if (exec_pipe(buffer_redirect->pipe_fd) == -1) {
         debug(1, PIPE_ERROR);
@@ -93,8 +94,7 @@ io_buffer_t *io_buffer_t::create(int fd, const io_chain_t &conflicts) {
     }
 
     if (!success) {
-        delete buffer_redirect;
-        buffer_redirect = NULL;
+        buffer_redirect.reset();
     }
     return buffer_redirect;
 }
@@ -139,21 +139,21 @@ void io_print(const io_chain_t &chain)
 {
     if (chain.empty())
     {
-        fprintf(stderr, "Empty chain %p\n", &chain);
+        fwprintf(stderr, L"Empty chain %p\n", &chain);
         return;
     }
 
-    fprintf(stderr, "Chain %p (%ld items):\n", &chain, (long)chain.size());
+    fwprintf(stderr, L"Chain %p (%ld items):\n", &chain, (long)chain.size());
     for (size_t i=0; i < chain.size(); i++)
     {
         const shared_ptr<io_data_t> &io = chain.at(i);
         if (io.get() == NULL)
         {
-            fprintf(stderr, "\t(null)\n");
+            fwprintf(stderr, L"\t(null)\n");
         }
         else
         {
-            fprintf(stderr, "\t%lu: fd:%d, ", (unsigned long)i, io->fd);
+            fwprintf(stderr, L"\t%lu: fd:%d, ", (unsigned long)i, io->fd);
             io->print();
         }
     }
@@ -165,35 +165,35 @@ void io_print(const io_chain_t &chain)
 /// created is marked close-on-exec. Returns -1 on failure (in which case the given fd is still
 /// closed).
 static int move_fd_to_unused(int fd, const io_chain_t &io_chain) {
-    int new_fd = fd;
-    if (fd >= 0 && io_chain.get_io_for_fd(fd).get() != NULL) {
-        // We have fd >= 0, and it's a conflict. dup it and recurse. Note that we recurse before
-        // anything is closed; this forces the kernel to give us a new one (or report fd
-        // exhaustion).
-        int tmp_fd;
-        do {
-            tmp_fd = dup(fd);
-        } while (tmp_fd < 0 && errno == EINTR);
-
-        assert(tmp_fd != fd);
-        if (tmp_fd < 0) {
-            // Likely fd exhaustion.
-            new_fd = -1;
-        } else {
-            // Ok, we have a new candidate fd. Recurse. If we get a valid fd, either it's the same
-            // as what we gave it, or it's a new fd and what we gave it has been closed. If we get a
-            // negative value, the fd also has been closed.
-            set_cloexec(tmp_fd);
-            new_fd = move_fd_to_unused(tmp_fd, io_chain);
-        }
-
-        // We're either returning a new fd or an error. In both cases, we promise to close the old
-        // one.
-        assert(new_fd != fd);
-        int saved_errno = errno;
-        exec_close(fd);
-        errno = saved_errno;
+    if (fd < 0 || io_chain.get_io_for_fd(fd).get() == NULL) {
+        return fd;
     }
+
+    // We have fd >= 0, and it's a conflict. dup it and recurse. Note that we recurse before
+    // anything is closed; this forces the kernel to give us a new one (or report fd exhaustion).
+    int new_fd = fd;
+    int tmp_fd;
+    do {
+        tmp_fd = dup(fd);
+    } while (tmp_fd < 0 && errno == EINTR);
+
+    assert(tmp_fd != fd);
+    if (tmp_fd < 0) {
+        // Likely fd exhaustion.
+        new_fd = -1;
+    } else {
+        // Ok, we have a new candidate fd. Recurse. If we get a valid fd, either it's the same as
+        // what we gave it, or it's a new fd and what we gave it has been closed. If we get a
+        // negative value, the fd also has been closed.
+        set_cloexec(tmp_fd);
+        new_fd = move_fd_to_unused(tmp_fd, io_chain);
+    }
+
+    // We're either returning a new fd or an error. In both cases, we promise to close the old one.
+    assert(new_fd != fd);
+    int saved_errno = errno;
+    exec_close(fd);
+    errno = saved_errno;
     return new_fd;
 }
 
