@@ -4,12 +4,12 @@
 #include "config.h"  // IWYU pragma: keep
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <wchar.h>
 
+#include <memory>
 #include <string>
 #include <type_traits>
 #include <vector>
@@ -26,10 +26,10 @@
 
 static bool path_get_path_core(const wcstring &cmd, wcstring *out_path,
                                const env_var_t &bin_path_var) {
-    int err = ENOENT;
     debug(3, L"path_get_path( '%ls' )", cmd.c_str());
 
-    // If the command has a slash, it must be a full path.
+    // If the command has a slash, it must be an absolute or relative path and thus we don't bother
+    // looking for a matching command.
     if (cmd.find(L'/') != wcstring::npos) {
         if (waccess(cmd, X_OK) != 0) {
             return false;
@@ -47,6 +47,7 @@ static bool path_get_path_core(const wcstring &cmd, wcstring *out_path,
         return false;
     }
 
+    int err = ENOENT;
     wcstring bin_path;
     if (!bin_path_var.missing()) {
         bin_path = bin_path_var;
@@ -56,7 +57,7 @@ static bool path_get_path_core(const wcstring &cmd, wcstring *out_path,
         // for the fish programs. Possibly with a duplicate dir if PREFIX is empty, "/", "/usr" or
         // "/usr/". If the PREFIX duplicates /bin or /usr/bin that is harmless other than a trivial
         // amount of time testing a path we've already tested.
-        bin_path = L"/bin" ARRAY_SEP_STR L"/usr/bin" ARRAY_SEP_STR PREFIX L"/bin";
+        bin_path = *list_to_array_val(wcstring_list_t({L"/bin", L"/usr/bin", PREFIX L"/bin"}));
     }
 
     std::vector<wcstring> pathsv;
@@ -85,6 +86,13 @@ static bool path_get_path_core(const wcstring &cmd, wcstring *out_path,
                 case ENOTDIR: {
                     break;
                 }
+                //WSL has a bug where access(2) can return EINVAL
+                //See https://github.com/Microsoft/BashOnWindows/issues/2522
+                //The only other way EINVAL can happen is if the wrong
+                //mode was specified, but we have X_OK hard-coded above.
+                case EINVAL: {
+                    break;
+                }
                 default: {
                     debug(1, MISSING_COMMAND_ERR_MSG, next_path.c_str());
                     wperror(L"access");
@@ -106,6 +114,40 @@ bool path_get_path(const wcstring &cmd, wcstring *out_path) {
     return path_get_path_core(cmd, out_path, env_get_string(L"PATH"));
 }
 
+wcstring_list_t path_get_paths(const wcstring &cmd) {
+    debug(3, L"path_get_paths('%ls')", cmd.c_str());
+    wcstring_list_t paths;
+
+    // If the command has a slash, it must be an absolute or relative path and thus we don't bother
+    // looking for matching commands in the PATH var.
+    if (cmd.find(L'/') != wcstring::npos) {
+        struct stat buff;
+        if (wstat(cmd, &buff)) return paths;
+        if (!S_ISREG(buff.st_mode)) return paths;
+        if (waccess(cmd, X_OK)) return paths;
+        paths.push_back(cmd);
+        return paths;
+    }
+
+    wcstring env_path = env_get_string(L"PATH");
+    std::vector<wcstring> pathsv;
+    tokenize_variable_array(env_path, pathsv);
+    for (auto path : pathsv) {
+        if (path.empty()) continue;
+        append_path_component(path, cmd);
+        if (waccess(path, X_OK) == 0) {
+            struct stat buff;
+            if (wstat(path, &buff) == -1) {
+                if (errno != EACCES) wperror(L"stat");
+                continue;
+            }
+            if (S_ISREG(buff.st_mode)) paths.push_back(path);
+        }
+    }
+
+    return paths;
+}
+
 bool path_get_cdpath(const wcstring &dir, wcstring *out, const wchar_t *wd,
                      const env_vars_snapshot_t &env_vars) {
     int err = ENOENT;
@@ -240,7 +282,7 @@ static void maybe_issue_path_warning(const wcstring &which_dir, const wcstring &
         debug(0, _(L"The error was '%s'."), strerror(saved_errno));
         debug(0, _(L"Please set $%ls to a directory where you have write access."), env_var);
     }
-    fputwc(L'\n', stderr);
+    ignore_result(write(STDERR_FILENO, "\n", 1));
 }
 
 static void path_create(wcstring &path, const wcstring &xdg_var, const wcstring &which_dir,
