@@ -16,9 +16,7 @@ void print_help(const char *c, int fd) {
     char cmd[CMD_LEN];
     int printed = snprintf(cmd, CMD_LEN, "fish -c '__fish_print_help %s >&%d'", c, fd);
 
-    if (printed < CMD_LEN) {
-        if ((system(cmd) == -1)) {
-            write_loop(2, HELP_ERR, strlen(HELP_ERR));
-        }
+    if (printed < CMD_LEN && system(cmd) == -1) {
+        write_loop(2, HELP_ERR, strlen(HELP_ERR));
     }
 }
