@@ -7,10 +7,14 @@
 // IWYU pragma: no_include <cstddef>
 #include "config.h"
 
+#include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
+#include <wchar.h>
+
 #if HAVE_NCURSES_H
 #include <ncurses.h>
 #elif HAVE_NCURSES_CURSES_H
@@ -23,9 +27,7 @@
 #elif HAVE_NCURSES_TERM_H
 #include <ncurses/term.h>
 #endif
-#include <assert.h>
-#include <time.h>
-#include <wchar.h>
+
 #include <algorithm>
 #include <string>
 #include <vector>
@@ -48,6 +50,8 @@
 /// A helper value for an invalid location.
 #define INVALID_LOCATION (screen_data_t::cursor_t(-1, -1))
 
+enum prompt_type_t { UNKNOWN_PROMPT, LEFT_PROMPT, RIGHT_PROMPT };
+
 static void invalidate_soft_wrap(screen_t *scr);
 
 /// Ugly kludge. The internal buffer used to store output of tputs. Since tputs external function
@@ -75,6 +79,9 @@ class scoped_buffer_t {
     }
 };
 
+// Singleton of the cached escape sequences seen in prompts and similar strings.
+cached_esc_sequences_t cached_esc_sequences = cached_esc_sequences_t();
+
 /// Tests if the specified narrow character sequence is present at the specified position of the
 /// specified wide character string. All of \c seq must match, but str may be longer than seq.
 static size_t try_sequence(const char *seq, const wchar_t *str) {
@@ -83,7 +90,8 @@ static size_t try_sequence(const char *seq, const wchar_t *str) {
         if (seq[i] != str[i]) return 0;
     }
 
-    return 0;
+    DIE("unexpectedly fell off end of try_sequence()");
+    return 0;  // this should never be executed
 }
 
 /// Returns the number of columns left until the next tab stop, given the current cursor postion.
@@ -105,24 +113,29 @@ static bool allow_soft_wrap(void) {
 
 /// Does this look like the escape sequence for setting a screen name.
 static bool is_screen_name_escape_seq(const wchar_t *code, size_t *resulting_length) {
-    bool found = false;
-    if (code[1] == L'k') {
-        const env_var_t term_name = env_get_string(L"TERM");
-        if (!term_name.missing() && string_prefixes_string(L"screen", term_name)) {
-            const wchar_t *const screen_name_end_sentinel = L"\x1b\\";
-            const wchar_t *screen_name_end = wcsstr(&code[2], screen_name_end_sentinel);
-            if (screen_name_end == NULL) {
-                // Consider just <esc>k to be the code.
-                *resulting_length = 2;
-            } else {
-                const wchar_t *escape_sequence_end =
-                    screen_name_end + wcslen(screen_name_end_sentinel);
-                *resulting_length = escape_sequence_end - code;
-            }
-            found = true;
-        }
+    if (code[1] != L'k') {
+        return false;
     }
-    return found;
+
+#if 0
+    // TODO: Decide if this should be removed or modified to also test for TERM values that begin
+    // with "tmux". See issue #3512.
+    const env_var_t term_name = env_get_string(L"TERM");
+    if (term_name.missing() || !string_prefixes_string(L"screen", term_name)) {
+        return false;
+    }
+#endif
+
+    const wchar_t *const screen_name_end_sentinel = L"\e\\";
+    const wchar_t *screen_name_end = wcsstr(&code[2], screen_name_end_sentinel);
+    if (screen_name_end == NULL) {
+        // Consider just <esc>k to be the code.
+        *resulting_length = 2;
+    } else {
+        const wchar_t *escape_sequence_end = screen_name_end + wcslen(screen_name_end_sentinel);
+        *resulting_length = escape_sequence_end - code;
+    }
+    return true;
 }
 
 /// iTerm2 escape codes: CSI followed by ], terminated by either BEL or escape + backslash.
@@ -134,7 +147,7 @@ static bool is_iterm2_escape_seq(const wchar_t *code, size_t *resulting_length)
         size_t cursor = 2;
         for (; code[cursor] != L'\0'; cursor++) {
             // Consume a sequence of characters up to <esc>\ or <bel>.
-            if (code[cursor] == '\x07' || (code[cursor] == '\\' && code[cursor - 1] == '\x1b')) {
+            if (code[cursor] == '\x07' || (code[cursor] == '\\' && code[cursor - 1] == '\e')) {
                 found = true;
                 break;
             }
@@ -169,123 +182,137 @@ static bool is_two_byte_escape_seq(const wchar_t *code, size_t *resulting_length
 /// Generic VT100 CSI-style sequence. <esc>, followed by zero or more ASCII characters NOT in
 /// the range [@,_], followed by one character in that range.
 static bool is_csi_style_escape_seq(const wchar_t *code, size_t *resulting_length) {
-    bool found = false;
-    if (code[1] == L'[') {
-        // Start at 2 to skip over <esc>[
-        size_t cursor = 2;
-        for (; code[cursor] != L'\0'; cursor++) {
-            // Consume a sequence of ASCII characters not in the range [@, ~].
-            wchar_t widechar = code[cursor];
-
-            // If we're not in ASCII, just stop.
-            if (widechar > 127) break;
-
-            // If we're the end character, then consume it and then stop.
-            if (widechar >= L'@' && widechar <= L'~') {
-                cursor++;
-                break;
-            }
-        }
-        // curs now indexes just beyond the end of the sequence (or at the terminating zero).
-        found = true;
-        *resulting_length = cursor;
+    if (code[1] != L'[') {
+        return false;
     }
-    return found;
+
+    // Start at 2 to skip over <esc>[
+    size_t cursor = 2;
+    for (; code[cursor] != L'\0'; cursor++) {
+        // Consume a sequence of ASCII characters not in the range [@, ~].
+        wchar_t widechar = code[cursor];
+
+        // If we're not in ASCII, just stop.
+        if (widechar > 127) break;
+
+        // If we're the end character, then consume it and then stop.
+        if (widechar >= L'@' && widechar <= L'~') {
+            cursor++;
+            break;
+        }
+    }
+    // cursor now indexes just beyond the end of the sequence (or at the terminating zero).
+    *resulting_length = cursor;
+    return true;
 }
 
-/// Returns the number of characters in the escape code starting at 'code' (which should initially
-/// contain \x1b).
+/// Detect whether the escape sequence sets foreground/background color. Note that 24-bit color
+/// sequences are detected by `is_csi_style_escape_seq()` if they use the ANSI X3.64 pattern for
+/// such sequences. This function only handles those escape sequences for setting color that rely on
+/// the terminfo definition and which might use a different pattern.
+static bool is_color_escape_seq(const wchar_t *code, size_t *resulting_length) {
+    if (!cur_term) return false;
+
+    // Detect these terminfo color escapes with parameter value up to max_colors, all of which
+    // don't move the cursor.
+    char *const esc[] = {
+        set_a_foreground, set_a_background, set_foreground, set_background,
+    };
+
+    for (size_t p = 0; p < sizeof esc / sizeof *esc; p++) {
+        if (!esc[p]) continue;
+
+        for (int k = 0; k < max_colors; k++) {
+            size_t esc_seq_len = try_sequence(tparm(esc[p], k), code);
+            if (esc_seq_len) {
+                *resulting_length = esc_seq_len;
+                return true;
+            }
+        }
+    }
+
+    return false;
+}
+
+/// Detect whether the escape sequence sets one of the terminal attributes that affects how text is
+/// displayed other than the color.
+static bool is_visual_escape_seq(const wchar_t *code, size_t *resulting_length) {
+    if (!cur_term) return false;
+    char *const esc2[] = {
+        enter_bold_mode,      exit_attribute_mode,    enter_underline_mode,  exit_underline_mode,
+        enter_standout_mode,  exit_standout_mode,     flash_screen,          enter_subscript_mode,
+        exit_subscript_mode,  enter_superscript_mode, exit_superscript_mode, enter_blink_mode,
+        enter_italics_mode,   exit_italics_mode,      enter_reverse_mode,    enter_shadow_mode,
+        exit_shadow_mode,     enter_standout_mode,    exit_standout_mode,    enter_secure_mode,
+        enter_dim_mode,       enter_blink_mode,       enter_protected_mode,  enter_alt_charset_mode,
+        exit_alt_charset_mode};
+
+    for (size_t p = 0; p < sizeof esc2 / sizeof *esc2; p++) {
+        if (!esc2[p]) continue;
+        // Test both padded and unpadded version, just to be safe. Most versions of tparm don't
+        // actually seem to do anything these days.
+        size_t esc_seq_len = maxi(try_sequence(tparm(esc2[p]), code), try_sequence(esc2[p], code));
+        if (esc_seq_len) {
+            *resulting_length = esc_seq_len;
+            return true;
+        }
+    }
+
+    return false;
+}
+
+/// Returns the number of characters in the escape code starting at 'code'. We only handle sequences
+/// that begin with \e. If it doesn't we return zero. We also return zero if we don't recognize the
+/// escape sequence based on querying terminfo and other heuristics.
 size_t escape_code_length(const wchar_t *code) {
     assert(code != NULL);
+    if (*code != L'\e') return 0;
 
-    // The only escape codes we recognize start with \x1b.
-    if (code[0] != L'\x1b') return 0;
+    size_t esc_seq_len = cached_esc_sequences.find_entry(code);
+    if (esc_seq_len) return esc_seq_len;
 
-    size_t resulting_length = 0;
-    bool found = false;
-
-    if (cur_term != NULL) {
-        // Detect these terminfo color escapes with parameter value up to max_colors-1, all of which
-        // don't move
-        // the cursor.
-        char *const esc[] = {
-            set_a_foreground, set_a_background, set_foreground, set_background,
-        };
-
-        for (size_t p = 0; p < sizeof esc / sizeof *esc && !found; p++) {
-            if (!esc[p]) continue;
-
-            for (short k = 0; k < max_colors; k++) {
-                size_t len = try_sequence(tparm(esc[p], k), code);
-                if (len) {
-                    resulting_length = len;
-                    found = true;
-                    break;
-                }
-            }
-        }
-
-        // Detect these semi-common terminfo escapes without any parameter values, all of which
-        // don't move the cursor.
-        char *const esc2[] = {enter_bold_mode,        exit_attribute_mode,   enter_underline_mode,
-                              exit_underline_mode,    enter_standout_mode,   exit_standout_mode,
-                              flash_screen,           enter_subscript_mode,  exit_subscript_mode,
-                              enter_superscript_mode, exit_superscript_mode, enter_blink_mode,
-                              enter_italics_mode,     exit_italics_mode,     enter_reverse_mode,
-                              enter_shadow_mode,      exit_shadow_mode,      enter_standout_mode,
-                              exit_standout_mode,     enter_secure_mode,     enter_dim_mode,
-                              enter_blink_mode,       enter_protected_mode,  enter_alt_charset_mode,
-                              exit_alt_charset_mode};
-
-        for (size_t p = 0; p < sizeof esc2 / sizeof *esc2 && !found; p++) {
-            if (!esc2[p]) continue;
-            // Test both padded and unpadded version, just to be safe. Most versions of tparm don't
-            // actually seem to do anything these days.
-
-            size_t len = maxi(try_sequence(tparm(esc2[p]), code), try_sequence(esc2[p], code));
-            if (len) {
-                resulting_length = len;
-                found = true;
-            }
-        }
-    }
-
-    if (!found) found = is_screen_name_escape_seq(code, &resulting_length);
-    if (!found) found = is_iterm2_escape_seq(code, &resulting_length);
-    if (!found) found = is_single_byte_escape_seq(code, &resulting_length);
-    if (!found) found = is_csi_style_escape_seq(code, &resulting_length);
-    if (!found) is_two_byte_escape_seq(code, &resulting_length);
-
-    return resulting_length;
+    bool found = is_color_escape_seq(code, &esc_seq_len);
+    if (!found) found = is_visual_escape_seq(code, &esc_seq_len);
+    if (!found) found = is_screen_name_escape_seq(code, &esc_seq_len);
+    if (!found) found = is_iterm2_escape_seq(code, &esc_seq_len);
+    if (!found) found = is_single_byte_escape_seq(code, &esc_seq_len);
+    if (!found) found = is_csi_style_escape_seq(code, &esc_seq_len);
+    if (!found) found = is_two_byte_escape_seq(code, &esc_seq_len);
+    if (found) cached_esc_sequences.add_entry(code, esc_seq_len);
+    return esc_seq_len;
 }
 
-// Information about a prompt layout.
+// Information about the layout of a prompt.
 struct prompt_layout_t {
-    // How many lines the prompt consumes.
-    size_t line_count;
-    // Width of the longest line.
-    size_t max_line_width;
-    // Width of the last line.
-    size_t last_line_width;
+    size_t line_count;       // how many lines the prompt consumes
+    size_t max_line_width;   // width of the longest line
+    size_t last_line_width;  // width of the last line
 };
 
+// These are used by `calc_prompt_layout()` to avoid redundant calculations.
+static const wchar_t *cached_left_prompt = wcsdup(L"");
+static const wchar_t *cached_right_prompt = wcsdup(L"");
+static prompt_layout_t cached_left_prompt_layout = {1, 0, 0};
+static prompt_layout_t cached_right_prompt_layout = {1, 0, 0};
+
 /// Calculate layout information for the given prompt. Does some clever magic to detect common
-/// escape sequences that may be embeded in a prompt, such as color codes.
-static prompt_layout_t calc_prompt_layout(const wchar_t *prompt) {
+/// escape sequences that may be embeded in a prompt, such as those to set visual attributes.
+static prompt_layout_t calc_prompt_layout(const wchar_t *prompt, prompt_type_t which_prompt) {
+    if (which_prompt == LEFT_PROMPT && wcscmp(cached_left_prompt, prompt) == 0) {
+        return cached_left_prompt_layout;
+    }
+    if (which_prompt == RIGHT_PROMPT && wcscmp(cached_right_prompt, prompt) == 0) {
+        return cached_right_prompt_layout;
+    }
+
+    prompt_layout_t prompt_layout = {1, 0, 0};
     size_t current_line_width = 0;
-    size_t j;
 
-    prompt_layout_t prompt_layout = {};
-    prompt_layout.line_count = 1;
-
-    for (j = 0; prompt[j]; j++) {
-        if (prompt[j] == L'\x1b') {
-            // This is the start of an escape code. Skip over it if it's at least one character
-            // long.
-            size_t escape_len = escape_code_length(&prompt[j]);
-            if (escape_len > 0) {
-                j += escape_len - 1;
-            }
+    for (int j = 0; prompt[j]; j++) {
+        if (prompt[j] == L'\e') {
+            // This is the start of an escape code. Skip over it if it's at least one char long.
+            size_t len = escape_code_length(&prompt[j]);
+            if (len > 0) j += len - 1;
         } else if (prompt[j] == L'\t') {
             current_line_width = next_tab_stop(current_line_width);
         } else if (prompt[j] == L'\n' || prompt[j] == L'\f') {
@@ -296,13 +323,25 @@ static prompt_layout_t calc_prompt_layout(const wchar_t *prompt) {
         } else if (prompt[j] == L'\r') {
             current_line_width = 0;
         } else {
-            // Ordinary decent character. Just add width. This returns -1 for a control character -
-            // don't add that.
+            // Ordinary char. Add its width with care to ignore control chars which have width -1.
             current_line_width += fish_wcwidth_min_0(prompt[j]);
-            prompt_layout.max_line_width = maxi(prompt_layout.max_line_width, current_line_width);
+            if (current_line_width > prompt_layout.max_line_width) {
+                prompt_layout.max_line_width = current_line_width;
+            }
         }
     }
+
     prompt_layout.last_line_width = current_line_width;
+    if (which_prompt == LEFT_PROMPT) {
+        free((void *)cached_left_prompt);
+        cached_left_prompt = wcsdup(prompt);
+        cached_left_prompt_layout = prompt_layout;
+    }
+    if (which_prompt == RIGHT_PROMPT) {
+        free((void *)cached_right_prompt);
+        cached_right_prompt = wcsdup(prompt);
+        cached_right_prompt_layout = prompt_layout;
+    }
     return prompt_layout;
 }
 
@@ -312,7 +351,7 @@ static size_t calc_prompt_lines(const wcstring &prompt) {
     // calc_prompt_width_and_lines.
     size_t result = 1;
     if (prompt.find(L'\n') != wcstring::npos || prompt.find(L'\f') != wcstring::npos) {
-        result = calc_prompt_layout(prompt.c_str()).line_count;
+        result = calc_prompt_layout(prompt.c_str(), UNKNOWN_PROMPT).line_count;
     }
     return result;
 }
@@ -411,7 +450,7 @@ static void s_desired_append_char(screen_t *s, wchar_t b, int c, int indent, siz
         if ((s->desired.cursor.x + cw) > screen_width) {
             // Current line is soft wrapped (assuming we support it).
             s->desired.line(s->desired.cursor.y).is_soft_wrapped = true;
-            // fprintf(stderr, "\n\n1 Soft wrapping %d\n\n", s->desired.cursor.y);
+            // fwprintf(stderr, L"\n\n1 Soft wrapping %d\n\n", s->desired.cursor.y);
 
             line_no = (int)s->desired.line_count();
             s->desired.add_line();
@@ -514,7 +553,7 @@ static void s_move(screen_t *s, data_buffer_t *b, int new_x, int new_y) {
     // Note that this is required to avoid some visual glitches in iTerm (issue #1448).
     bool use_multi =
         multi_str != NULL && multi_str[0] != '\0' && abs(x_steps) * strlen(str) > strlen(multi_str);
-    if (use_multi) {
+    if (use_multi && cur_term) {
         char *multi_param = tparm(multi_str, abs(x_steps));
         writembs(multi_param);
     } else {
@@ -656,7 +695,7 @@ static bool test_stuff(screen_t *scr)
         int c = getchar();
         if (c != EOF) break;
     }
-    puts("Bye");
+    fwprintf(stdout, L"Bye\n");
     exit(0);
     while (1) sleep(10000);
     return true;
@@ -666,8 +705,9 @@ static bool test_stuff(screen_t *scr)
 /// Update the screen to match the desired output.
 static void s_update(screen_t *scr, const wchar_t *left_prompt, const wchar_t *right_prompt) {
     // if (test_stuff(scr)) return;
-    const size_t left_prompt_width = calc_prompt_layout(left_prompt).last_line_width;
-    const size_t right_prompt_width = calc_prompt_layout(right_prompt).last_line_width;
+    const size_t left_prompt_width = calc_prompt_layout(left_prompt, LEFT_PROMPT).last_line_width;
+    const size_t right_prompt_width =
+        calc_prompt_layout(right_prompt, RIGHT_PROMPT).last_line_width;
 
     int screen_width = common_get_width();
 
@@ -817,7 +857,7 @@ static void s_update(screen_t *scr, const wchar_t *left_prompt, const wchar_t *r
         }
 
         // Output any rprompt if this is the first line.
-        if (i == 0 && right_prompt_width > 0) {
+        if (i == 0 && right_prompt_width > 0) {  //!OCLINT(Use early exit/continue)
             s_move(scr, &output, (int)(screen_width - right_prompt_width), (int)i);
             s_set_color(scr, &output, 0xffffffff);
             s_write_str(&output, right_prompt);
@@ -861,7 +901,10 @@ static void s_update(screen_t *scr, const wchar_t *left_prompt, const wchar_t *r
 }
 
 /// Returns true if we are using a dumb terminal.
-static bool is_dumb(void) { return !cursor_up || !cursor_down || !cursor_left || !cursor_right; }
+static bool is_dumb(void) {
+    if (!cur_term) return true;
+    return !cursor_up || !cursor_down || !cursor_left || !cursor_right;
+}
 
 struct screen_layout_t {
     // The left prompt that we're going to use.
@@ -903,8 +946,8 @@ static screen_layout_t compute_layout(screen_t *s, size_t screen_width,
     const wchar_t *right_prompt = right_prompt_str.c_str();
     const wchar_t *autosuggestion = autosuggestion_str.c_str();
 
-    prompt_layout_t left_prompt_layout = calc_prompt_layout(left_prompt);
-    prompt_layout_t right_prompt_layout = calc_prompt_layout(right_prompt);
+    prompt_layout_t left_prompt_layout = calc_prompt_layout(left_prompt, LEFT_PROMPT);
+    prompt_layout_t right_prompt_layout = calc_prompt_layout(right_prompt, RIGHT_PROMPT);
 
     size_t left_prompt_width = left_prompt_layout.last_line_width;
     size_t right_prompt_width = right_prompt_layout.last_line_width;
@@ -977,44 +1020,52 @@ static screen_layout_t compute_layout(screen_t *s, size_t screen_width,
     // the right prompt? The issue is resizing: if you resize the window smaller, then the right
     // prompt will wrap to the next line. This means that we can't go back to the line that we were
     // on, and things turn to chaos very quickly.
+    size_t calculated_width;
     bool done = false;
 
     // Case 1
-    if (!done &&
-        left_prompt_width + right_prompt_width + first_command_line_width +
-                autosuggest_total_width <
-            screen_width) {
-        result.left_prompt = left_prompt;
-        result.left_prompt_space = left_prompt_width;
-        result.right_prompt = right_prompt;
-        result.autosuggestion = autosuggestion;
-        done = true;
+    if (!done) {
+        calculated_width = left_prompt_width + right_prompt_width + first_command_line_width +
+                           autosuggest_total_width;
+        if (calculated_width < screen_width) {
+            result.left_prompt = left_prompt;
+            result.left_prompt_space = left_prompt_width;
+            result.right_prompt = right_prompt;
+            result.autosuggestion = autosuggestion;
+            done = true;
+        }
     }
 
     // Case 2. Note that we require strict inequality so that there's always at least one space
     // between the left edge and the rprompt.
-    if (!done && left_prompt_width + right_prompt_width + first_command_line_width < screen_width) {
-        result.left_prompt = left_prompt;
-        result.left_prompt_space = left_prompt_width;
-        result.right_prompt = right_prompt;
+    if (!done) {
+        calculated_width = left_prompt_width + right_prompt_width + first_command_line_width;
+        if (calculated_width < screen_width) {
+            result.left_prompt = left_prompt;
+            result.left_prompt_space = left_prompt_width;
+            result.right_prompt = right_prompt;
 
-        // Need at least two characters to show an autosuggestion.
-        size_t available_autosuggest_space =
-            screen_width - (left_prompt_width + right_prompt_width + first_command_line_width);
-        if (autosuggest_total_width > 0 && available_autosuggest_space > 2) {
-            size_t truncation_offset = truncation_offset_for_width(autosuggest_truncated_widths,
-                                                                   available_autosuggest_space - 2);
-            result.autosuggestion = wcstring(autosuggestion, truncation_offset);
-            result.autosuggestion.push_back(ellipsis_char);
+            // Need at least two characters to show an autosuggestion.
+            size_t available_autosuggest_space =
+                screen_width - (left_prompt_width + right_prompt_width + first_command_line_width);
+            if (autosuggest_total_width > 0 && available_autosuggest_space > 2) {
+                size_t truncation_offset = truncation_offset_for_width(
+                    autosuggest_truncated_widths, available_autosuggest_space - 2);
+                result.autosuggestion = wcstring(autosuggestion, truncation_offset);
+                result.autosuggestion.push_back(ellipsis_char);
+            }
+            done = true;
         }
-        done = true;
     }
 
     // Case 3
-    if (!done && left_prompt_width + first_command_line_width < screen_width) {
-        result.left_prompt = left_prompt;
-        result.left_prompt_space = left_prompt_width;
-        done = true;
+    if (!done) {
+        calculated_width = left_prompt_width + first_command_line_width;
+        if (calculated_width < screen_width) {
+            result.left_prompt = left_prompt;
+            result.left_prompt_space = left_prompt_width;
+            done = true;
+        }
     }
 
     // Case 4
@@ -1029,11 +1080,8 @@ static screen_layout_t compute_layout(screen_t *s, size_t screen_width,
             prompt_percent_width > 33) {
             result.prompts_get_own_line = true;
         }
-
-        done = true;
     }
 
-    assert(done);
     return result;
 }
 
@@ -1190,9 +1238,10 @@ void s_reset(screen_t *s, screen_reset_mode_t mode) {
         // Don't need to check for fish_wcwidth errors; this is done when setting up
         // omitted_newline_char in common.cpp.
         int non_space_width = fish_wcwidth(omitted_newline_char);
-        if (screen_width >= non_space_width) {
+        // We do `>` rather than `>=` because the code below might require one extra space.
+        if (screen_width > non_space_width) {
             bool justgrey = true;
-            if (enter_dim_mode) {
+            if (cur_term && enter_dim_mode) {
                 std::string dim = tparm(enter_dim_mode);
                 if (!dim.empty()) {
                     // Use dim if they have it, so the color will be based on their actual normal
@@ -1201,7 +1250,7 @@ void s_reset(screen_t *s, screen_reset_mode_t mode) {
                     justgrey = false;
                 }
             }
-            if (justgrey && set_a_foreground) {
+            if (cur_term && justgrey && set_a_foreground) {
                 if (max_colors >= 238) {
                     // draw the string in a particular grey
                     abandon_line_string.append(str2wcstring(tparm(set_a_foreground, 237)));
@@ -1214,22 +1263,31 @@ void s_reset(screen_t *s, screen_reset_mode_t mode) {
                     abandon_line_string.append(str2wcstring(tparm(set_a_foreground, 0)));
                 }
             }
+
             abandon_line_string.push_back(omitted_newline_char);
 
-            if (exit_attribute_mode) {
+            if (cur_term && exit_attribute_mode) {
                 abandon_line_string.append(
                     str2wcstring(tparm(exit_attribute_mode)));  // normal text ANSI escape sequence
             }
-            abandon_line_string.append(screen_width - non_space_width, L' ');
+
+            int newline_glitch_width = term_has_xn ? 0 : 1;
+            abandon_line_string.append(screen_width - non_space_width - newline_glitch_width, L' ');
         }
+
         abandon_line_string.push_back(L'\r');
+        abandon_line_string.push_back(omitted_newline_char);
         // Now we are certainly on a new line. But we may have dropped the omitted newline char on
         // it. So append enough spaces to overwrite the omitted newline char, and then clear all the
-        // spaces from the new line
+        // spaces from the new line.
         abandon_line_string.append(non_space_width, L' ');
         abandon_line_string.push_back(L'\r');
-        // clear entire line - el2
-        abandon_line_string.append(L"\x1b[2K");
+        // Clear entire line. Zsh doesn't do this. Fish added this with commit 4417a6ee: If you have
+        // a prompt preceded by a new line, you'll get a line full of spaces instead of an empty
+        // line above your prompt. This doesn't make a difference in normal usage, but copying and
+        // pasting your terminal log becomes a pain. This commit clears that line, making it an
+        // actual empty line.
+        abandon_line_string.append(L"\e[2K");
 
         const std::string narrow_abandon_line_string = wcs2string(abandon_line_string);
         write_loop(STDOUT_FILENO, narrow_abandon_line_string.c_str(),
