@@ -2,14 +2,14 @@
 // extended to support marks, tokenizing multiple strings and disposing of unused string segments.
 #include "config.h"  // IWYU pragma: keep
 
-#include <assert.h>
 #include <fcntl.h>
 #include <limits.h>
-#include <stdlib.h>
 #include <unistd.h>
 #include <wchar.h>
 #include <wctype.h>
+
 #include <string>
+#include <type_traits>
 
 #include "common.h"
 #include "fallback.h"  // IWYU pragma: keep
@@ -64,10 +64,10 @@ tokenizer_t::tokenizer_t(const wchar_t *b, tok_flags_t flags)
       continue_line_after_comment(false) {
     assert(b != NULL);
 
-    this->accept_unfinished = !!(flags & TOK_ACCEPT_UNFINISHED);
-    this->show_comments = !!(flags & TOK_SHOW_COMMENTS);
-    this->squash_errors = !!(flags & TOK_SQUASH_ERRORS);
-    this->show_blank_lines = !!(flags & TOK_SHOW_BLANK_LINES);
+    this->accept_unfinished = static_cast<bool>(flags & TOK_ACCEPT_UNFINISHED);
+    this->show_comments = static_cast<bool>(flags & TOK_SHOW_COMMENTS);
+    this->squash_errors = static_cast<bool>(flags & TOK_SQUASH_ERRORS);
+    this->show_blank_lines = static_cast<bool>(flags & TOK_SHOW_BLANK_LINES);
 
     this->has_next = (*b != L'\0');
     this->tok_next();
@@ -219,6 +219,7 @@ void tokenizer_t::read_string() {
                             if (!tok_is_string_character(*(this->buff), is_first)) {
                                 do_loop = 0;
                             }
+                            break;
                         }
                     }
                     break;
@@ -265,6 +266,9 @@ void tokenizer_t::read_string() {
                             do_loop = 0;
                             break;
                         }
+                        default: {
+                            break;  // ignore other chars
+                        }
                     }
                     break;
                 }
@@ -285,6 +289,9 @@ void tokenizer_t::read_string() {
                             do_loop = 0;
                             break;
                         }
+                        default: {
+                            break;  // ignore other chars
+                        }
                     }
                     break;
                 }
@@ -318,7 +325,7 @@ void tokenizer_t::read_string() {
                 break;
             }
             default: {
-                assert(0 && "Unexpected mode in read_string");
+                DIE("unexpected mode in read_string");
                 break;
             }
         }
@@ -483,7 +490,7 @@ void tokenizer_t::tok_next() {
     }
 
     if (!this->has_next) {
-        // wprintf( L"EOL\n" );
+        // fwprintf(stdout, L"EOL\n" );
         this->last_type = TOK_END;
         return;
     }
@@ -603,7 +610,7 @@ wcstring tok_first(const wcstring &str) {
     tokenizer_t t(str.c_str(), TOK_SQUASH_ERRORS);
     tok_t token;
     if (t.next(&token) && token.type == TOK_STRING) {
-        result.swap(token.text);
+        result = std::move(token.text);
     }
     return result;
 }
@@ -660,7 +667,7 @@ bool move_word_state_machine_t::consume_char_path_components(wchar_t c) {
         s_end
     };
 
-    // printf("state %d, consume '%lc'\n", state, c);
+    // fwprintf(stdout, L"state %d, consume '%lc'\n", state, c);
     bool consumed = false;
     while (state != s_end && !consumed) {
         switch (state) {
@@ -706,10 +713,7 @@ bool move_word_state_machine_t::consume_char_path_components(wchar_t c) {
                 break;
             }
             case s_end:
-            default: {
-                // We won't get here, but keep the compiler happy.
-                break;
-            }
+            default: { break; }
         }
     }
     return consumed;
@@ -760,8 +764,9 @@ bool move_word_state_machine_t::consume_char(wchar_t c) {
         case move_word_style_whitespace: {
             return consume_char_whitespace(c);
         }
-        default: { abort(); }
     }
+
+    DIE("should not reach this statement");  // silence some compiler errors about not returning
 }
 
 move_word_state_machine_t::move_word_state_machine_t(move_word_style_t syl)
